"""Class implementation for append line point mix-in.
"""

from typing import Union

from typing_extensions import final

from apysc._display.points_2d_mixin import Points2DMixIn
from apysc._display.set_x_and_y_with_minimum_point_interface_base import (
    SetXAndYWithMinimumPointInterfaceBase,
)
from apysc._html.debug_mode import add_debug_info_setting
from apysc._type.int import Int
from apysc._type.variable_name_suffix_mixin import VariableNameSuffixMixIn
from apysc._validation import arg_validation_decos


class AppendLinePointMixIn(Points2DMixIn, VariableNameSuffixMixIn):

    _points_var_name: str

    @final
    @arg_validation_decos.is_integer(arg_position_index=1)
    @arg_validation_decos.is_integer(arg_position_index=2)
    @add_debug_info_setting(module_name=__name__)
    def append_line_point(self, *, x: Union[int, Int], y: Union[int, Int]) -> None:
        """
        Append line point at the end.

        Parameters
        ----------
        x : int or Int
            X-coordinate.
        y : int or Int
            Y-coordinate.

        Examples
        --------
        >>> import apysc as ap
        >>> stage: ap.Stage = ap.Stage()
        >>> sprite: ap.Sprite = ap.Sprite()
        >>> sprite.graphics.begin_fill(color="#0af")
        >>> polygon: ap.Polygon = sprite.graphics.draw_polygon(
        ...     points=[
        ...         ap.Point2D(x=0, y=0),
        ...         ap.Point2D(x=0, y=50),
        ...         ap.Point2D(x=50, y=50),
        ...     ]
        ... )
        >>> polygon.append_line_point(x=50, y=0)
        """
        import apysc as ap
        from apysc._type import value_util

        if not hasattr(self, "_points_var_name"):
            raise AttributeError(
                "_points_var_name attribute is not set. Please add "
                "implementation to set that value when constructor "
                "or else."
            )
        suffix: str = self._get_attr_variable_name_suffix(attr_identifier="points")
        point: ap.Point2D = ap.Point2D(x=x, y=y, variable_name_suffix=suffix)
        self.points.append(value=point)
        expression: str
        x_name: str = value_util.get_value_str_for_expression(value=x)
        y_name: str = value_util.get_value_str_for_expression(value=y)
        expression = (
            f"{self._points_var_name}.push([{x_name}, {y_name}]);"
            f"\n{self.variable_name}.plot({self._points_var_name});"
        )
        ap.append_js_expression(expression=expression)

        if isinstance(self, SetXAndYWithMinimumPointInterfaceBase):
            self._set_x_and_y_with_minimum_point()



"""This module is for the constants definitions of
the expression's variable names.
"""

from typing_extensions import Final

DOCUMENT: Final[str] = "document"
BLANK_OBJECT: Final[str] = "bo"
DISPLAY_OBJECT: Final[str] = "do"
PARENT: Final[str] = "parent"
GRAPHICS: Final[str] = "g"
POINT2D: Final[str] = "p2d"
RECTANGLE: Final[str] = "rect"
CIRCLE: Final[str] = "circle"
ELLIPSE: Final[str] = "ellipse"
LINE: Final[str] = "line"
POLYLINE: Final[str] = "pline"
POLYGON: Final[str] = "poly"
PATH: Final[str] = "path"
SPRITE: Final[str] = "sp"
ARRAY: Final[str] = "arr"
DICTIONARY: Final[str] = "dct"
INDEX: Final[str] = "idx"
BOOLEAN: Final[str] = "b"
INT: Final[str] = "i"
NUMBER: Final[str] = "n"
STRING: Final[str] = "s"
SNAPSHOT: Final[str] = "snapshot"
ANY: Final[str] = "any"
EVENT: Final[str] = "evt"
MOUSE_EVENT: Final[str] = "mevt"
WHEEL_EVENT: Final[str] = "wevt"
TIMER_EVENT: Final[str] = "tevt"
ANIMATION_EVENT: Final[str] = "aevt"
ENTER_FRAME_EVENT: Final[str] = "eevt"
TIMER: Final[str] = "timer"
DATETIME: Final[str] = "dt"
ANIMATION_MOVE: Final[str] = "animmove"
ANIMATION_X: Final[str] = "animx"
ANIMATION_Y: Final[str] = "animy"
ANIMATION_CX: Final[str] = "animcx"
ANIMATION_CY: Final[str] = "animcy"
ANIMATION_WIDTH: Final[str] = "animwidth"
ANIMATION_HEIGHT: Final[str] = "animheight"
ANIMATION_WIDTH_FOR_ELLIPSE: Final[str] = "animwidthforellipse"
ANIMATION_HEIGHT_FOR_ELLIPSE: Final[str] = "animheightforellipse"
ANIMATION_FILL_ALPHA: Final[str] = "animfillalpha"
ANIMATION_FILL_COLOR: Final[str] = "animfillcolor"
ANIMATION_LINE_COLOR: Final[str] = "animlinecolor"
ANIMATION_LINE_ALPHA: Final[str] = "animlinealpha"
ANIMATION_LINE_THICKNESS: Final[str] = "animlinethickness"
ANIMATION_RADIUS: Final[str] = "animradius"
ANIMATION_ROTATION_AROUND_CENTER: Final[str] = "animrotardcenter"
ANIMATION_ROTATION_AROUND_POINT: Final[str] = "animrotardpoint"
ANIMATION_SCALE_X_FROM_CENTER: Final[str] = "animscalexfromcenter"
ANIMATION_SCALE_Y_FROM_CENTER: Final[str] = "animscaleyfromcenter"
ANIMATION_SCALE_X_FROM_POINT: Final[str] = "animscalexfrompoint"
ANIMATION_SCALE_Y_FROM_POINT: Final[str] = "animscaleyfrompoint"
ANIMATION_PARALLEL: Final[str] = "animparallel"



"""Implementations of the Path class.
"""

from typing import List
from typing import Optional
from typing import Union

from typing_extensions import final

from apysc._display import graphics
from apysc._display.child_mixin import ChildMixIn
from apysc._display.fill_alpha_mixin import FillAlphaMixIn
from apysc._display.fill_color_mixin import FillColorMixIn
from apysc._display.graphics_base import GraphicsBase
from apysc._display.line_caps import LineCaps
from apysc._display.line_dash_dot_setting import LineDashDotSetting
from apysc._display.line_dash_setting import LineDashSetting
from apysc._display.line_dot_setting import LineDotSetting
from apysc._display.line_joints import LineJoints
from apysc._display.line_round_dot_setting import LineRoundDotSetting
from apysc._display.x_mixin import XMixIn
from apysc._display.y_mixin import YMixIn
from apysc._geom.path_data_base import PathDataBase
from apysc._html.debug_mode import add_debug_info_setting
from apysc._type.int import Int
from apysc._type.number import Number
from apysc._type.string import String
from apysc._type.variable_name_suffix_mixin import VariableNameSuffixMixIn
from apysc._validation import arg_validation_decos


class Path(
    XMixIn,
    YMixIn,
    GraphicsBase,
    FillColorMixIn,
    FillAlphaMixIn,
    VariableNameSuffixMixIn,
):
    """
    The path vector graphics class.

    References
    ----------
    - Path class
        - https://simon-ritchie.github.io/apysc/en/path.html
    - Graphics draw_path interface
        - https://simon-ritchie.github.io/apysc/en/graphics_draw_path.html
    - PathMoveTo class
        - https://simon-ritchie.github.io/apysc/en/path_move_to.html
    - PathLineTo class
        - https://simon-ritchie.github.io/apysc/en/path_line_to.html
    - PathHorizontal class
        - https://simon-ritchie.github.io/apysc/en/path_horizontal.html
    - PathVertical class
        - https://simon-ritchie.github.io/apysc/en/path_vertical.html
    - PathClose class
        - https://simon-ritchie.github.io/apysc/en/path_close.html
    - PathBezier2D class
        - https://simon-ritchie.github.io/apysc/en/path_bezier_2d.html
    - PathBezier2DContinual class
        - https://simon-ritchie.github.io/apysc/en/path_bezier_2d_continual.html
    - PathBezier3D class
        - https://simon-ritchie.github.io/apysc/en/path_bezier_3d.html
    - PathBezier3DContinual class
        - https://simon-ritchie.github.io/apysc/en/path_bezier_3d_continual.html

    Examples
    --------
    >>> import apysc as ap
    >>> stage: ap.Stage = ap.Stage()
    >>> sprite: ap.Sprite = ap.Sprite()
    >>> sprite.graphics.line_style(color="#fff", thickness=3)
    >>> path: ap.Path = sprite.graphics.draw_path(
    ...     path_data_list=[
    ...         ap.PathMoveTo(x=0, y=50),
    ...         ap.PathBezier2D(control_x=50, control_y=0, dest_x=100, dest_y=50),
    ...     ]
    ... )
    """

    _path_data_list: List[PathDataBase]

    # self
    @arg_validation_decos.multiple_line_settings_are_not_set(arg_position_index=0)
    # path_data_list
    @arg_validation_decos.is_valid_path_data_list(arg_position_index=1)
    # fill_color
    @arg_validation_decos.is_hex_color_code_format(arg_position_index=2)
    # fill_alpha
    @arg_validation_decos.num_is_0_to_1_range(arg_position_index=3)
    # line_color
    @arg_validation_decos.is_hex_color_code_format(arg_position_index=4)
    # line_alpha
    @arg_validation_decos.num_is_0_to_1_range(arg_position_index=5)
    # line_thickness
    @arg_validation_decos.is_integer(arg_position_index=6)
    @arg_validation_decos.num_is_gte_zero(arg_position_index=6)
    # line_cap
    @arg_validation_decos.is_line_cap(arg_position_index=7, optional=True)
    # line_joints
    @arg_validation_decos.is_line_joints(arg_position_index=8, optional=True)
    # line_dot_setting
    @arg_validation_decos.is_line_dot_setting(arg_position_index=9)
    # line_dash_setting
    @arg_validation_decos.is_line_dash_setting(arg_position_index=10)
    # line_round_dot_setting
    @arg_validation_decos.is_line_round_dot_setting(arg_position_index=11)
    # line_dash_dot_setting
    @arg_validation_decos.is_line_dash_dot_setting(arg_position_index=12)
    # parent
    @arg_validation_decos.is_display_object_container(
        arg_position_index=13, optional=True
    )
    # variable_name_suffix
    @arg_validation_decos.is_builtin_string(arg_position_index=14, optional=False)
    @add_debug_info_setting(module_name=__name__)
    def __init__(
        self,
        *,
        path_data_list: List[PathDataBase],
        fill_color: Union[str, String] = "",
        fill_alpha: Union[float, Number] = 1.0,
        line_color: Union[str, String] = "",
        line_alpha: Union[float, Number] = 1.0,
        line_thickness: Union[int, Int] = 1,
        line_cap: Optional[Union[String, LineCaps]] = None,
        line_joints: Optional[Union[String, LineJoints]] = None,
        line_dot_setting: Optional[LineDotSetting] = None,
        line_dash_setting: Optional[LineDashSetting] = None,
        line_round_dot_setting: Optional[LineRoundDotSetting] = None,
        line_dash_dot_setting: Optional[LineDashDotSetting] = None,
        parent: Optional[ChildMixIn] = None,
        variable_name_suffix: str = "",
    ) -> None:
        """
        Create a path vector graphic.

        Parameters
        ----------
        path_data_list : list of PathDataBase
            Target path data settings, such as the ap.PathData.MoveTo.
        fill_color : str or String, default ''
            A fill-color to set.
        fill_alpha : float or Number, default 1.0
            A fill-alpha to set.
        line_color : str or String, default ''
            A line-color to set.
        line_alpha : float or Number, default 1.0
            A line-alpha to set.
        line_thickness : int or Int, default 1
            A line-thickness (line-width) to set.
        line_cap : String or LineCaps or None, default None
            A line-cap setting to set.
        line_joints : String or LineJoints or None, default None
            A line-joints setting to set.
        line_dot_setting : LineDotSetting or None, default None
            A dot setting to set.
        line_dash_setting : LineDashSetting or None, default None
            A dash setting to set.
        line_round_dot_setting : LineRoundDotSetting or None, default None
            A round-dot setting to set.
        line_dash_dot_setting : LineDashDotSetting or None, default None
            A dash dot (1-dot chain) setting to set.
        parent : ChildMixIn or None, default None
            A parent instance to add this instance.
            If a specified value is None, this interface uses
            a stage instance.
        variable_name_suffix : str, default ''
            A JavaScript variable name suffix string.
            This setting is sometimes useful for JavaScript's debugging.

        References
        ----------
        - Path class
            - https://simon-ritchie.github.io/apysc/en/path.html
        - Graphics draw_path interface
            - https://simon-ritchie.github.io/apysc/en/graphics_draw_path.html
        - PathMoveTo class
            - https://simon-ritchie.github.io/apysc/en/path_move_to.html
        - PathLineTo class
            - https://simon-ritchie.github.io/apysc/en/path_line_to.html
        - PathHorizontal class
            - https://simon-ritchie.github.io/apysc/en/path_horizontal.html
        - PathVertical class
            - https://simon-ritchie.github.io/apysc/en/path_vertical.html
        - PathClose class
            - https://simon-ritchie.github.io/apysc/en/path_close.html
        - PathBezier2D class
            - https://simon-ritchie.github.io/apysc/en/path_bezier_2d.html
        - PathBezier2DContinual class
            - https://simon-ritchie.github.io/apysc/en/path_bezier_2d_continual.html
        - PathBezier3D class
            - https://simon-ritchie.github.io/apysc/en/path_bezier_3d.html
        - PathBezier3DContinual class
            - https://simon-ritchie.github.io/apysc/en/path_bezier_3d_continual.html

        Examples
        --------
        >>> import apysc as ap
        >>> stage: ap.Stage = ap.Stage()
        >>> path: ap.Path = ap.Path(
        ...     path_data_list=[
        ...         ap.PathMoveTo(x=0, y=50),
        ...         ap.PathBezier2D(control_x=50, control_y=0, dest_x=100, dest_y=50),
        ...     ],
        ...     line_color="#ffffff",
        ...     line_thickness=3,
        ... )
        >>> path.line_color
        String('#ffffff')
        >>> path.line_thickness
        Int(3)
        """
        from apysc._expression import expression_variables_util
        from apysc._expression import var_names

        self._variable_name_suffix = variable_name_suffix
        variable_name: str = expression_variables_util.get_next_variable_name(
            type_name=var_names.PATH
        )
        self.variable_name = variable_name
        self._set_initial_basic_values(
            fill_color=fill_color,
            fill_alpha=fill_alpha,
            line_color=line_color,
            line_thickness=line_thickness,
            line_alpha=line_alpha,
            line_cap=line_cap,
            line_joints=line_joints,
        )
        self._path_data_list = path_data_list
        self._append_constructor_expression()
        self._set_line_setting_if_not_none_value_exists(
            line_dot_setting=line_dot_setting,
            line_dash_setting=line_dash_setting,
            line_round_dot_setting=line_round_dot_setting,
            line_dash_dot_setting=line_dash_dot_setting,
        )
        super(Path, self).__init__(parent=parent, variable_name=variable_name)

    @classmethod
    @final
    def _create_with_graphics(
        cls,
        *,
        graphics: "graphics.Graphics",
        path_data_list: List[PathDataBase],
        variable_name_suffix: str = "",
    ) -> "Path":
        """
        Create a path instance with the instance of
        specified graphics.

        Parameters
        ----------
        graphics : Graphics
            Graphics instance to link this instance.
        path_data_list : list of PathDataBase
            Target path data settings, such as the ap.PathData.MoveTo.
        variable_name_suffix : str, default ''
            A JavaScript variable name suffix string.
            This setting is sometimes useful for JavaScript's debugging.

        Returns
        -------
        path : Path
            A created path instance.
        """
        path: Path = Path(
            path_data_list=path_data_list,
            fill_color=graphics._fill_color,
            fill_alpha=graphics._fill_alpha,
            line_color=graphics._line_color,
            line_alpha=graphics._line_alpha,
            line_thickness=graphics._line_thickness,
            line_cap=graphics._line_cap,
            line_joints=graphics._line_joints,
            line_dot_setting=graphics._line_dot_setting,
            line_dash_setting=graphics._line_dash_setting,
            line_round_dot_setting=graphics._line_round_dot_setting,
            line_dash_dot_setting=graphics._line_dash_dot_setting,
            parent=graphics,
            variable_name_suffix=variable_name_suffix,
        )
        return path

    @final
    @add_debug_info_setting(module_name=__name__)
    def _append_constructor_expression(self) -> None:
        """
        Append a constructor expression.
        """
        import apysc as ap
        from apysc._geom.path_data_util import make_paths_expression_from_list
        from apysc._string import indent_util

        stage: ap.Stage = ap.get_stage()
        path_data_expression: str = make_paths_expression_from_list(
            path_data_list=self._path_data_list
        )
        INDENT_NUM: int = 2
        expression: str = (
            f"var {self.variable_name} = {stage.variable_name}"
            f"\n  .path({path_data_expression})"
            "\n  .attr({"
        )
        expression = self._append_basic_vals_expression(
            expression=expression, indent_num=INDENT_NUM
        )
        spaces: str = indent_util.make_spaces_for_html(indent_num=INDENT_NUM)
        if self._fill_color._value == "":
            expression += f'\n{spaces}fill: "transparent",'
        expression += "\n  });"
        ap.append_js_expression(expression=expression)

    def __repr__(self) -> str:
        """
        Get a string representation of this instance (for the sake of
        debugging).

        Returns
        -------
        repr_str : str
            Type name and variable name will be set
            (e.g., `Path('<variable_name>')`).
        """
        repr_str: str = f"Path('{self.variable_name}')"
        return repr_str



"""This module is for adding document translations mapping
dictionary's blank data.
"""

import os
import re
from typing import Dict
from typing import List
from typing import Match
from typing import Optional
from typing import Pattern
from typing import Union

from apysc._lint_and_doc import document_text_split_util
from apysc._lint_and_doc import lint_and_doc_hash_util
from apysc._lint_and_doc.docs_lang import Lang
from apysc._lint_and_doc.document_text_split_util import BodyText
from apysc._lint_and_doc.document_text_split_util import CodeBlock
from apysc._lint_and_doc.document_text_split_util import Heading
from apysc._lint_and_doc.lint_and_doc_hash_util import HashType
from apysc._lint_and_doc.translation_mapping_utils import MAPPING_CONST_NAME
from apysc._lint_and_doc.translation_mapping_utils import MAPPING_UNNECESSARY_PATTERNS
from apysc._lint_and_doc.translation_mapping_utils import SKIPPING_PATTERNS
from apysc._lint_and_doc.translation_mapping_utils import (
    convert_splitted_values_to_keys,
)
from apysc._lint_and_doc.translation_mapping_utils import escape_key_or_value
from apysc._lint_and_doc.translation_mapping_utils import get_hash_type_from_lang
from apysc._lint_and_doc.translation_mapping_utils import get_mapping_module_path
from apysc._lint_and_doc.translation_mapping_utils import (
    get_translated_file_path_from_src_path,
)
from apysc._lint_and_doc.translation_mapping_utils import read_mapping_data
from apysc._lint_and_doc.translation_mapping_utils import remove_empty_keys
from apysc._lint_and_doc.translation_mapping_utils import (
    remove_escaping_from_key_or_value,
)

_SplittedVals = List[Union[Heading, BodyText, CodeBlock]]

_NO_MAPPING_FIXED_STRINGS: List[str] = [
    "</details>",
    "<hr>",
    "- ValueError: ",
    "- Exception: ",
    "---",
]


def add_mapping_blank_data(*, lang: Lang) -> None:
    """
    Add a mapping's blank data of document translations.

    Parameters
    ----------
    lang : Lang
        A target translation language.
    """
    from apysc._file import file_util
    from apysc._lint_and_doc.fixed_translation_mapping import data_model

    is_fixed_mapping_updated: bool = data_model.is_fixed_mapping_updated(lang=lang)

    src_docs_file_paths: List[str] = _get_src_docs_file_paths()
    hash_type: HashType = get_hash_type_from_lang(lang=lang)
    if not is_fixed_mapping_updated:
        src_docs_file_paths = lint_and_doc_hash_util.remove_not_updated_file_paths(
            file_paths=src_docs_file_paths, hash_type=hash_type
        )
    for src_doc_file_path in src_docs_file_paths:
        markdown_txt: str = file_util.read_txt(file_path=src_doc_file_path)
        splitted_values: _SplittedVals = (
            document_text_split_util.split_markdown_document(markdown_txt=markdown_txt)
        )
        keys: List[str] = convert_splitted_values_to_keys(
            splitted_values=splitted_values
        )
        keys = remove_empty_keys(keys=keys)
        keys = _remove_skipping_pattern_keys_from_list(keys=keys)
        mappings: List[Dict[str, str]] = _make_mappings_from_keys(
            keys=keys, src_doc_file_path=src_doc_file_path, lang=lang
        )
        _save_mapping_data(
            mappings=mappings, src_doc_file_path=src_doc_file_path, lang=lang
        )
    lint_and_doc_hash_util.save_target_files_hash(
        file_paths=src_docs_file_paths, hash_type=hash_type
    )
    data_model.save_fixed_mapping_hash(lang=lang)


def _save_mapping_data(
    *, mappings: List[Dict[str, str]], src_doc_file_path: str, lang: Lang
) -> None:
    """
    Save mapping data module with specified mappings data.

    Parameters
    ----------
    mappings : list of dict
        Target mappings data to save.
    src_doc_file_path : str
        A target source document's file path.
    lang : Lang
        A target translation language.
    """
    from apysc._file import file_util

    basename: str = os.path.basename(src_doc_file_path)
    module_str: str = (
        f'"""This module is for the translation mapping data '
        "of the\nfollowing document:"
        f"\n\nDocument file: {basename}"
        f"\nLanguage: {lang.value}"
        '\n"""'
        "\n\nfrom typing import Dict"
        f"\n\n{MAPPING_CONST_NAME}: Dict[str, str] = {{"
    )
    for mapping in mappings:
        module_str += "\n    "
        module_str += "#" * 50
        key: str = list(mapping.keys())[0]
        value: str = list(mapping.values())[0]
        line: str = f'    "{key}": "{value}",'
        if len(line) >= 88:
            line += "  # noqa"
        module_str += f"\n{line}"
    module_str += "\n}\n"
    module_path: str = get_mapping_module_path(
        src_doc_file_path=src_doc_file_path, lang=lang
    )
    file_util.save_plain_txt(txt=module_str, file_path=module_path)


def _make_mappings_from_keys(
    *, keys: List[str], src_doc_file_path: str, lang: Lang
) -> List[Dict[str, str]]:
    """
    Make mapping dictionary values from specified key values.

    Parameters
    ----------
    keys : list of str
        Target dictionary key values.
    src_doc_file_path : str
        A target source document file path.
    lang : Lang
        A target translation language.

    Returns
    -------
    mappings : list of dict
        Created mapping dictionary values. A dictionary
        value becomes a blank string if it is a new mapping value.
    """
    already_saved_mapping: Dict[str, str] = read_mapping_data(
        src_doc_file_path=src_doc_file_path, lang=lang
    )
    mappings: List[Dict[str, str]] = []
    for key in keys:
        key_: str = remove_escaping_from_key_or_value(key_or_val=key)
        value: str = already_saved_mapping.get(key_, "")
        value = escape_key_or_value(key_or_val=value)
        value = _set_fixed_translation_value_if_exists(key=key_, value=value)
        value = _set_same_value_if_code_block_mapping_is_blank(key=key, value=value)
        value = _set_translated_file_names_to_toc_code_block(lang=lang, value=value)
        value = _convert_link_list_by_lang(key=key, value=value, lang=lang)
        value = _set_same_value_if_api_params_or_returns_list(key=key, value=value)
        value = _set_same_value_if_key_is_no_mapping_fixed_string(key=key, value=value)
        value = _set_same_value_if_key_is_image_link(key=key, value=value)
        key = escape_key_or_value(key_or_val=key)
        value = escape_key_or_value(key_or_val=value)
        mappings.append({key: value})
    return mappings


def _set_translated_file_names_to_toc_code_block(*, lang: Lang, value: str) -> str:
    """
    Set translated file names to a specified mapping value
    if a value is the table of contents' code block.

    Parameters
    ----------
    lang : Lang
        A target language setting.
    value : str
        A mapping value.

    Returns
    -------
    value : str
        A result mapping value.
    """
    if not value.startswith("```{toctree}"):
        return value
    value = remove_escaping_from_key_or_value(key_or_val=value)
    lines: List[str] = value.splitlines()
    result_lines: List[str] = []
    for line in lines:
        match: Optional[Match] = re.match(
            pattern=r"^[a-z_0-9]+$",
            string=line,
        )
        if match is None:
            result_lines.append(line)
            continue
        if line.startswith(f"{lang.value}_"):
            result_lines.append(line)
            continue
        result_lines.append(f"{lang.value}_{line}")
    value = "\n".join(result_lines)
    return value


_BLANK_ALT_IMAGE_LINK_PATTERN: Pattern = re.compile(pattern=r"^\!\[\]\(.+?\)$")


def _set_same_value_if_key_is_image_link(*, key: str, value: str) -> str:
    """
    Set the same key as a value if a specified key is an
    image link string with an alternative blank text.

    Parameters
    ----------
    key : str
        A target key string.
    value : str
        A target value.

    Returns
    -------
    value : str
        This interface returns the same value if a specified
        key is an image link string with an alternative blank
        text. Otherwise, it returns a value argument directly.
    """
    match: Optional[Match] = _BLANK_ALT_IMAGE_LINK_PATTERN.match(string=key)
    if match is None:
        return value
    return key


_API_DOC_PARAMS_OR_RETURNS_NAME_PATTERN: Pattern = re.compile(
    pattern=r"^\- \`.+?\`\: .*$"
)


def _set_same_value_if_key_is_no_mapping_fixed_string(*, key: str, value: str) -> str:
    """
    Set the same key as a value if a specified key is a
    no-mapping fixed string.

    Parameters
    ----------
    key : str
        A target key string.
    value : str
        A target value.

    Returns
    -------
    value : str
        This interface returns a key's value if a specified
        key is a no-mapping fixed string. Otherwise, this
        interface returns a specified value directly.
    """
    for no_mapping_fixed_string in _NO_MAPPING_FIXED_STRINGS:
        if key == no_mapping_fixed_string:
            return key
    return value


def _set_same_value_if_api_params_or_returns_list(*, key: str, value: str) -> str:
    """
    Set the same key as a value if a specified key string
    is an API's parameters or returns list's line.

    Parameters
    ----------
    key : str
        A target key string.
    value : str
        A target value.

    Returns
    -------
    value : str
        This interface returns a key's value if a specified
        key string is an API's parameters or returns line.
        Otherwise, this interface returns a specified value directly.
    """
    match: Optional[Match] = _API_DOC_PARAMS_OR_RETURNS_NAME_PATTERN.match(string=key)
    if match is None:
        return value
    return key


_LINK_PATTERN: Pattern = re.compile(
    pattern=(r"(.*?\[.+?\])" r"\((.+?\.(md|html))\)" r"(?P<after_txt>.*?)"),
    flags=re.MULTILINE,
)


def _convert_link_list_by_lang(*, key: str, value: str, lang: Lang) -> str:
    """
    Convert a link list value by a specified language if a key's
    value is a link list.

    Parameters
    ----------
    key : str
        A target key string (source English string).
    value : str
        A target value string.
    lang : Lang
        A target translation language.

    Returns
    -------
    value : str
        A converted value. This interface skips the
        conversion if a specified key's value is not
        a link list.
    """
    if not key.startswith("- "):
        return value
    match: Optional[Match] = _LINK_PATTERN.search(string=key)
    if match is None:
        return value
    value = ""
    while match is not None:
        prev_txt: str = match.group(1)
        doc_path: str = match.group(2)
        doc_path = get_translated_file_path_from_src_path(
            source_doc_path=doc_path, lang=lang
        )
        if value != "":
            value += "\n"
        value += f"{prev_txt}({doc_path})"

        key = _LINK_PATTERN.sub(repl=r"\g<after_txt>", string=key, count=1)
        match = _LINK_PATTERN.search(string=key)
    value = _replace_link_text_by_fixed_mapping(value=value, lang=lang)
    value = _replace_link_en_dir_with_target_lang(value=value, lang=lang)
    value = escape_key_or_value(key_or_val=value)
    return value


def _replace_link_en_dir_with_target_lang(*, value: str, lang: Lang) -> str:
    """
    Replace a link's `/en/` directory path with a specified language directory path.

    Parameters
    ----------
    value : str
        A target markdown string value to replace.
    lang : Lang
        A target translation language.

    Returns
    -------
    value : str
        Replaced markdown string.
    """
    value = re.sub(
        pattern=r"\/en\/",
        repl=rf"/{lang.value}/",
        string=value,
    )
    return value


def _replace_link_text_by_fixed_mapping(*, value: str, lang: Lang) -> str:
    """
    Replace each link text if there are fixed translation-mapping
    settings.

    Parameters
    ----------
    value : str
        A link text.
    lang : Lang
        A target translation language.

    Returns
    -------
    value : str
        A replaced link text. This interface returns a blank string
        if there is no translation mapping.
    """
    from apysc._lint_and_doc.fixed_translation_mapping import data_model

    link_texts: List[str] = _extract_link_texts(value=value)
    for link_text in link_texts:
        translation_str: str = data_model.get_fixed_translation_str_if_exists(
            key=link_text, lang=lang
        )
        if translation_str == "":
            return ""
        value = value.replace(f"[{link_text}](", f"[{translation_str}](")
    return value


_LinkTextPattern: Pattern = re.compile(
    pattern=r".*?\[(.+?)\]\(.+?\.(md|html)\)", flags=re.MULTILINE
)


def _extract_link_texts(*, value: str) -> List[str]:
    """
    Extract link texts from a specified value string.

    Parameters
    ----------
    value : str
        A target value string.

    Returns
    -------
    link_texts : list of str
        Extracted link texts.
    """
    link_texts: List[str] = []
    match: Optional[Match] = _LinkTextPattern.search(string=value)
    while match is not None:
        link_text: str = match.group(1)
        link_texts.append(link_text)
        value = _LinkTextPattern.sub(repl="", string=value, count=1)
        match = _LinkTextPattern.search(string=value)
    return link_texts


def _set_same_value_if_code_block_mapping_is_blank(*, key: str, value: str) -> str:
    """
    Set the same code-block value if a specified value
    is a blank string and a key's value is a code block.

    Parameters
    ----------
    key : str
        A target key string (source English string).
    value : str
        A target value string.

    Returns
    -------
    value : str
        A result value. If an argument value is not a blank
        string, this interface returns it directly.
        If a key's value is not a code block, this interface
        also returns an argument value directly.
    """
    if value != "":
        return value
    if not key.startswith("```"):
        return value
    return key


def _set_fixed_translation_value_if_exists(*, key: str, value: str) -> str:
    """
    Set a fixed translation value if its mapping setting exists.

    Parameters
    ----------
    key : str
        A target translation key.
    value : str
        A current translation value.

    Returns
    -------
    fixed_value : str
        A fixed translation value. If there is no fixed
        translation mapping setting, this interface directly
        returns an argument's value.
    """
    from apysc._lint_and_doc.fixed_translation_mapping import data_model

    fixed_value: str = data_model.get_fixed_translation_str_if_exists(
        key=key, lang=Lang.JP
    )
    if fixed_value == "":
        return value
    return fixed_value


def _remove_skipping_pattern_keys_from_list(*, keys: List[str]) -> List[str]:
    """
    Remove skipping pattern matching keys from a specified list.

    Parameters
    ----------
    keys : list of str
        A target key's list.

    Returns
    -------
    result_keys : list of str
        An after removing key's list.
    """
    result_keys: List[str] = []
    patterns: List[Pattern] = [*SKIPPING_PATTERNS, *MAPPING_UNNECESSARY_PATTERNS]
    for key in keys:
        is_pattern_matching: bool = False
        for pattern in patterns:
            match: Optional[Match] = pattern.search(string=key)
            if match is not None:
                is_pattern_matching = True
                break
        if is_pattern_matching:
            continue
        result_keys.append(key)
    return result_keys


def _get_src_docs_file_paths() -> List[str]:
    """
    Get source documents file paths.

    Returns
    -------
    src_docs_file_paths : list of str
        Source documents file paths.
    """
    dir_path: str = "./docs_src/source/"
    file_or_dir_names: List[str] = os.listdir(dir_path)
    src_docs_file_paths: List[str] = []
    for file_or_dir_name in file_or_dir_names:
        file_or_dir_path: str = os.path.join(dir_path, file_or_dir_name)
        if not os.path.isfile(file_or_dir_path):
            continue
        if not file_or_dir_path.endswith(".md"):
            continue
        if _is_translated_document(path=file_or_dir_path):
            continue
        src_docs_file_paths.append(file_or_dir_path)
    return src_docs_file_paths


def _is_translated_document(*, path: str) -> bool:
    """
    Get a boolean indicating whether a specified document path
    is a translated document or not.

    Parameters
    ----------
    path : str
        A target document path.

    Returns
    -------
    result : bool
        If a specified document path is a translated document,
        this interface returns True.
    """
    basename: str = os.path.basename(path)
    for lang in Lang:
        if basename.startswith(lang.value):
            return True
    return False



"""This module is for the translation mapping data of the
following document:

Document file: animation_duration.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {
    ##################################################
    "# Animation interfaces duration setting": "# 各アニメーションのインターフェイスのduration設定",
    ##################################################
    "This page explains the animation interfaces `duration` setting.": "このページでは各アニメーションのインターフェイスの`duration`設定について説明します。",  # noqa
    ##################################################
    "## What setting is this?": "## 設定概要",
    ##################################################
    "The `duration` setting determines the animation time from start to end in milliseconds. For instance, if you specify 3000 to the `duration` argument, the animation takes 3 seconds to complete.": "`duration`設定はアニメーションの開始から終了までにかける時間を設定します。単位はミリ秒です。例えば`duration`引数に3000を指定したらアニメーションが完了するまで3秒必要とする設定になります。",  # noqa
    ##################################################
    "Each animation method interface (such as the `animation_move`\\, `animation_x`\\, and so on) has the `duration` argument.": "`animation_move`などの各アニメーションのインターフェイスはこの`duration`引数を持っています。",  # noqa
    ##################################################
    "## Basic usage": "## 基本的な使い方",
    ##################################################
    "The following example sets 3 seconds to the `duration` option and animates x-coordinate.": "以下の例ではX座標のアニメーションで`duration`の設定に3秒を設定しています。",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\nDURATION: int = 3000\nEASING: ap.Easing = ap.Easing.EASE_OUT_QUINT\n\n\ndef on_animation_complete_1(e: ap.AnimationEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that animation calls when its end.\n\n    Parameters\n    ----------\n    e : ap.AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    rectangle: ap.Rectangle = e.this.target\n    animation_x: ap.AnimationX = rectangle.animation_x(\n        x=50, duration=DURATION, easing=EASING\n    )\n    animation_x.animation_complete(on_animation_complete_2)\n    animation_x.start()\n\n\ndef on_animation_complete_2(e: ap.AnimationEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that animation calls when its end.\n\n    Parameters\n    ----------\n    e : ap.AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    rectangle: ap.Rectangle = e.this.target\n    animation_x: ap.AnimationX = rectangle.animation_x(\n        x=400, duration=DURATION, easing=EASING\n    )\n    animation_x.animation_complete(on_animation_complete_1)\n    animation_x.start()\n\n\nap.Stage(\n    stage_width=500, stage_height=150, background_color="#333", stage_elem_id="stage"\n)\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color="#0af")\nrectangle: ap.Rectangle = sprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\n\nanimation_x: ap.AnimationX = rectangle.animation_x(\n    x=400, duration=DURATION, easing=EASING\n)\nanimation_x.animation_complete(on_animation_complete_1)\nanimation_x.start()\n\nap.save_overall_html(dest_dir_path="./animation_duration_basic_usage/")\n```': '```py\n# runnable\nimport apysc as ap\n\nDURATION: int = 3000\nEASING: ap.Easing = ap.Easing.EASE_OUT_QUINT\n\n\ndef on_animation_complete_1(e: ap.AnimationEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that animation calls when its end.\n\n    Parameters\n    ----------\n    e : ap.AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    rectangle: ap.Rectangle = e.this.target\n    animation_x: ap.AnimationX = rectangle.animation_x(\n        x=50, duration=DURATION, easing=EASING\n    )\n    animation_x.animation_complete(on_animation_complete_2)\n    animation_x.start()\n\n\ndef on_animation_complete_2(e: ap.AnimationEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that animation calls when its end.\n\n    Parameters\n    ----------\n    e : ap.AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    rectangle: ap.Rectangle = e.this.target\n    animation_x: ap.AnimationX = rectangle.animation_x(\n        x=400, duration=DURATION, easing=EASING\n    )\n    animation_x.animation_complete(on_animation_complete_1)\n    animation_x.start()\n\n\nap.Stage(\n    stage_width=500, stage_height=150, background_color="#333", stage_elem_id="stage"\n)\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color="#0af")\nrectangle: ap.Rectangle = sprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\n\nanimation_x: ap.AnimationX = rectangle.animation_x(\n    x=400, duration=DURATION, easing=EASING\n)\nanimation_x.animation_complete(on_animation_complete_1)\nanimation_x.start()\n\nap.save_overall_html(dest_dir_path="./animation_duration_basic_usage/")\n```',  # noqa
}



"""This module is for the translation mapping data of the
following document:

Document file: index.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {
    ##################################################
    "# apysc documentation": "# apysc ドキュメント",
    ##################################################
    "Welcome to apysc documentation! apysc is the Python front-end library (currently developing and only works partially).": "apyscのドキュメントページへようこそ。apyscはPythonのフロントエンド用のライブラリです（現在開発中であり部分的にのみ動作します）。",  # noqa
    ##################################################
    "## Project links": "## プロジェクトの関連リンク",
    ##################################################
    "- [GitHub](https://github.com/simon-ritchie/apysc)": "- [GitHub](https://github.com/simon-ritchie/apysc)",  # noqa
    ##################################################
    "  - Please leave a ⭐️star⭐️ if you favor the apysc library or have high hopes for the library's future!": "  - もしもapyscライブラリを気にいったり、もしくはライブラリの将来に期待が出来そうと感じられたらリポジトリにスターを付けていただけますと幸いです。",  # noqa
    ##################################################
    "- [PyPI](https://pypi.org/project/apysc/)": "- [PyPI](https://pypi.org/project/apysc/)",  # noqa
    ##################################################
    "Other languages documentations: | English | [Japanese (日本語)](https://simon-ritchie.github.io/apysc/jp/jp_index.html) |": "他の言語のドキュメント: | [英語 (English)](https://simon-ritchie.github.io/apysc/en/index.html) | 日本語 |",  # noqa
    ##################################################
    "## Quick start guide": "## クイックスタートガイド",
    ##################################################
    "```{toctree}\n:maxdepth: 1\nwhat_apysc_can_do\nquick_start\nimport_conventions\nrecommended_type_checker_settings\n```": "```{toctree}\n:maxdepth: 1\njp_what_apysc_can_do\njp_quick_start\njp_import_conventions\njp_recommended_type_checker_settings\n```",  # noqa
    ##################################################
    "## Container classes": "## コンテナーの各クラス",
    ##################################################
    "The `Stage` is the apysc overall drawing area container, and the `Sprite` is the container class.": "`Stage`はapyscの描画領域全体のコンテナーとなり、`Sprite`は通常の各インスタンスのコンテナーのクラスとなります。",  # noqa
    ##################################################
    "```{toctree}\n:maxdepth: 1\nstage\nsprite\n```": "```{toctree}\n:maxdepth: 1\njp_stage\njp_sprite\n```",  # noqa
    ##################################################
    "## Exporting": "## 出力処理",
    ##################################################
    "The HTML and JavaScript exporting interfaces.": "以下はHTMLとJavaScriptの出力処理関係の各インターフェイスです。",  # noqa
    ##################################################
    "```{toctree}\n:maxdepth: 1\nsave_overall_html\ndisplay_on_jupyter\ndisplay_on_colaboratory\nappend_js_expression\n```": "```{toctree}\n:maxdepth: 1\njp_save_overall_html\njp_display_on_jupyter\njp_display_on_colaboratory\njp_append_js_expression\n```",  # noqa
    ##################################################
    "## Child-related interfaces": "## 子要素関係の各インターフェイス",
    ##################################################
    "The parent class, such as the `Sprite` or `Stage`, has the following interfaces:": "`Sprite`や`Stage`などの親となれる各クラスは以下のインターフェイスを持っています:",  # noqa
    ##################################################
    "```{toctree}\n:maxdepth: 1\nadd_child_and_remove_child\nremove_children\ncontains\nnum_children\nget_child_at\n```": "```{toctree}\n:maxdepth: 1\njp_add_child_and_remove_child\njp_remove_children\njp_contains\njp_num_children\njp_get_child_at\n```",  # noqa
    ##################################################
    "## apysc basic data classes": "## apyscの基本的な各データクラス",
    ##################################################
    "```{toctree}\n:maxdepth: 1\nwhy_apysc_doesnt_use_python_builtin_data_type\nfundamental_data_classes_value_interface\nint_and_number\nint_and_number_arithmetic_operations\nint_and_number_comparison_operations\nstring\nstring_comparison_operations\nstring_addition_and_multiplication\nboolean\narray\narray_append_and_push\narray_extend_and_concat\narray_insert_and_insert_at\narray_pop\narray_remove_and_remove_at\narray_sort\narray_reverse\narray_slice\narray_length\narray_join\narray_index_of\narray_clear\narray_comparison\ndictionary\ndictionary_generic\ndictionary_get\ndictionary_length\n```": "```{toctree}\n:maxdepth: 1\njp_why_apysc_doesnt_use_python_builtin_data_type\njp_fundamental_data_classes_value_interface\njp_int_and_number\njp_int_and_number_arithmetic_operations\njp_int_and_number_comparison_operations\njp_string\njp_string_comparison_operations\njp_string_addition_and_multiplication\njp_boolean\njp_array\njp_array_append_and_push\njp_array_extend_and_concat\njp_array_insert_and_insert_at\njp_array_pop\njp_array_remove_and_remove_at\njp_array_sort\njp_array_reverse\njp_array_slice\njp_array_length\njp_array_join\njp_array_index_of\njp_array_clear\njp_array_comparison\njp_dictionary\njp_dictionary_generic\njp_dictionary_get\njp_dictionary_length\n```",  # noqa
    ##################################################
    "## DisplayObject and GraphicsBase classes": "## DisplayObject と GraphicsBase の各クラス",  # noqa
    ##################################################
    "The `DisplayObject` class is the base class for each display object. The `GraphicsBase` class is the `DisplayObject` subclass and the base class of each graphic class, such as the `Rectangle` class.": "`DisplayObject`クラスは各表示用のオブジェクトの基底クラスです。`GraphicsBase`クラスは`DisplayObject`のサブクラスであり、且つ`Rectangle`などの各グラフィックスの基底クラスとなります。",  # noqa
    ##################################################
    "```{toctree}\n:maxdepth: 1\ndisplay_object\ndisplay_object_and_graphics_base_prop_abstract\ndisplay_object_x_and_y\ndisplay_object_parent\ndisplay_object_visible\ndisplay_object_get_and_set_css\ndisplay_object_mouse_event\ngraphics_base_fill_color\ngraphics_base_fill_alpha\ngraphics_base_line_color\ngraphics_base_line_alpha\ngraphics_base_line_thickness\ngraphics_base_line_dot_setting\ngraphics_base_line_dash_setting\ngraphics_base_line_round_dot_setting\ngraphics_base_line_dash_dot_setting\ngraphics_base_rotation_around_center\ngraphics_base_rotation_around_point\ngraphics_base_scale_from_center\ngraphics_base_scale_from_point\ngraphics_base_flip_interfaces\ngraphics_base_skew\n```": "```{toctree}\n:maxdepth: 1\njp_display_object\njp_display_object_and_graphics_base_prop_abstract\njp_display_object_x_and_y\njp_display_object_parent\njp_display_object_visible\njp_display_object_get_and_set_css\njp_display_object_mouse_event\njp_graphics_base_fill_color\njp_graphics_base_fill_alpha\njp_graphics_base_line_color\njp_graphics_base_line_alpha\njp_graphics_base_line_thickness\njp_graphics_base_line_dot_setting\njp_graphics_base_line_dash_setting\njp_graphics_base_line_round_dot_setting\njp_graphics_base_line_dash_dot_setting\njp_graphics_base_rotation_around_center\njp_graphics_base_rotation_around_point\njp_graphics_base_scale_from_center\njp_graphics_base_scale_from_point\njp_graphics_base_flip_interfaces\njp_graphics_base_skew\n```",  # noqa
    ##################################################
    "## Each graphic class": "## 各グラフィックのクラス",
    ##################################################
    "```{toctree}\n:maxdepth: 1\nrectangle\ncircle\nellipse\nline\npolyline\npolygon\npath\n```": "```{toctree}\n:maxdepth: 1\njp_rectangle\njp_circle\njp_ellipse\njp_line\njp_polyline\njp_polygon\njp_path\n```",  # noqa
    ##################################################
    "## Graphics class": "## Graphics クラス",
    ##################################################
    "The `Graphics` class handles each vector graphics drawing.": "`Graphics`クラスは各ベクターグラフィックスの描画処理を扱います。",  # noqa
    ##################################################
    "```{toctree}\n:maxdepth: 1\ndraw_interfaces_abstract\ngraphics\ngraphics_begin_fill\ngraphics_line_style\ngraphics_draw_rect\ngraphics_draw_round_rect\ngraphics_draw_circle\ngraphics_draw_ellipse\ngraphics_move_to_and_line_to\ngraphics_draw_line\ngraphics_draw_dotted_line\ngraphics_draw_dashed_line\ngraphics_draw_round_dotted_line\ngraphics_draw_dash_dotted_line\ngraphics_draw_polygon\ngraphics_draw_path\ngraphics_clear\n```": "```{toctree}\n:maxdepth: 1\njp_draw_interfaces_abstract\njp_graphics\njp_graphics_begin_fill\njp_graphics_line_style\njp_graphics_draw_rect\njp_graphics_draw_round_rect\njp_graphics_draw_circle\njp_graphics_draw_ellipse\njp_graphics_move_to_and_line_to\njp_graphics_draw_line\njp_graphics_draw_dotted_line\njp_graphics_draw_dashed_line\njp_graphics_draw_round_dotted_line\njp_graphics_draw_dash_dotted_line\njp_graphics_draw_polygon\njp_graphics_draw_path\njp_graphics_clear\n```",  # noqa
    ##################################################
    "## Geometry-related classes": "## 座標・サイズ等のデータの各クラス",
    ##################################################
    "```{toctree}\n:maxdepth: 1\npoint2d\npath_move_to\npath_line_to\npath_horizontal\npath_vertical\npath_close\npath_bezier_2d\npath_bezier_2d_continual\npath_bezier_3d\npath_bezier_3d_continual\n```": "```{toctree}\n:maxdepth: 1\njp_point2d\njp_path_move_to\njp_path_line_to\njp_path_horizontal\njp_path_vertical\njp_path_close\njp_path_bezier_2d\njp_path_bezier_2d_continual\njp_path_bezier_3d\njp_path_bezier_3d_continual\n```",  # noqa
    ##################################################
    "## Common event interfaces": "## イベントの共通の各インターフェイス",
    ##################################################
    "```{toctree}\n:maxdepth: 1\nabout_handler_options_type\nevent_prevent_default_and_stop_propagation\nbind_and_trigger_custom_event\n```": "```{toctree}\n:maxdepth: 1\njp_about_handler_options_type\njp_event_prevent_default_and_stop_propagation\njp_bind_and_trigger_custom_event\n```",  # noqa
    ##################################################
    "## MouseEvent class and mouse event binding": "## MouseEvent クラスとマウスイベントの設定",
    ##################################################
    "```{toctree}\n:maxdepth: 1\nmouse_event_abstract\nmouse_event_basic\nclick\ndblclick\nmousedown_and_mouseup\nmouseover_and_mouseout\nmousemove\n```": "```{toctree}\n:maxdepth: 1\njp_mouse_event_abstract\njp_mouse_event_basic\njp_click\njp_dblclick\njp_mousedown_and_mouseup\njp_mouseover_and_mouseout\njp_mousemove\n```",  # noqa
    ##################################################
    "## Branch instruction": "## 条件分岐の制御",
    ##################################################
    "```{toctree}\n:maxdepth: 1\nif\nelif\nelse\nbranch_instruction_variables_reverting_setting\nreturn\n```": "```{toctree}\n:maxdepth: 1\njp_if\njp_elif\njp_else\njp_branch_instruction_variables_reverting_setting\njp_return\n```",  # noqa
    ##################################################
    "## Loop": "## ループ",
    ##################################################
    "```{toctree}\n:maxdepth: 1\nfor\ncontinue\n```": "```{toctree}\n:maxdepth: 1\njp_for\njp_continue\n```",  # noqa
    ##################################################
    "## Timer": "## タイマー",
    ##################################################
    "```{toctree}\n:maxdepth: 1\ntimer\ntimer_event\ntimer_delay\nfps\ntimer_repeat_count\ntimer_start_and_stop\ntimer_complete\ntimer_reset\n```": "```{toctree}\n:maxdepth: 1\njp_timer\njp_timer_event\njp_timer_delay\njp_fps\njp_timer_repeat_count\njp_timer_start_and_stop\njp_timer_complete\njp_timer_reset\n```",  # noqa
    ##################################################
    "## DateTime": "## DateTime",
    ##################################################
    "```{toctree}\n:maxdepth: 1\ndatetime\ndatetime_year\ndatetime_month\ndatetime_day\ndatetime_hour\ndatetime_minute\ndatetime_second\ndatetime_millisecond\ndatetime_weekday_js_and_weekday_py\ndatetime_now\n```": "```{toctree}\n:maxdepth: 1\njp_datetime\njp_datetime_year\njp_datetime_month\njp_datetime_day\njp_datetime_hour\njp_datetime_minute\njp_datetime_second\njp_datetime_millisecond\njp_datetime_weekday_js_and_weekday_py\njp_datetime_now\n```",  # noqa
    ##################################################
    "## Animation": "## アニメーション",
    ##################################################
    "```{toctree}\n:maxdepth: 1\nanimation_interfaces_abstract\nanimation_event\nanimation_duration\nanimation_delay\nanimation_return_value\nanimation_base_start\nanimation_complete\nanimation_method_chaining\nanimation_base_target\nanimation_pause_and_play\nanimation_reset\nanimation_finish\nanimation_reverse\nanimation_time\neasing_enum\nsequential_animation\nanimation_parallel\nanimation_move\nanimation_x\nanimation_y\nanimation_width_and_height\nanimation_fill_color\nanimation_fill_alpha\nanimation_line_color\nanimation_line_alpha\nanimation_line_thickness\nanimation_radius\nanimation_rotation_around_center\nanimation_rotation_around_point\nanimation_scale_x_and_y_from_center\nanimation_scale_x_and_y_from_point\n```": "```{toctree}\n:maxdepth: 1\njp_animation_interfaces_abstract\njp_animation_event\njp_animation_duration\njp_animation_delay\njp_animation_return_value\njp_animation_base_start\njp_animation_complete\njp_animation_method_chaining\njp_animation_base_target\njp_animation_pause_and_play\njp_animation_reset\njp_animation_finish\njp_animation_reverse\njp_animation_time\njp_easing_enum\njp_sequential_animation\njp_animation_parallel\njp_animation_move\njp_animation_x\njp_animation_y\njp_animation_width_and_height\njp_animation_fill_color\njp_animation_fill_alpha\njp_animation_line_color\njp_animation_line_alpha\njp_animation_line_thickness\njp_animation_radius\njp_animation_rotation_around_center\njp_animation_rotation_around_point\njp_animation_scale_x_and_y_from_center\njp_animation_scale_x_and_y_from_point\n```",  # noqa
    ##################################################
    "## Other manipulation interfaces": "## その他の操作関係の各インターフェイス",
    ##################################################
    "```{toctree}\n:maxdepth: 1\ndelete\n```": "```{toctree}\n:maxdepth: 1\njp_delete\n```",  # noqa
    ##################################################
    "## Debugging": "## デバッグ",
    ##################################################
    "```{toctree}\n:maxdepth: 1\ntrace\nset_debug_mode\nunset_debug_mode\nadd_debug_info_setting\nvariable_name_suffix\n```": "```{toctree}\n:maxdepth: 1\njp_trace\njp_set_debug_mode\njp_unset_debug_mode\njp_add_debug_info_setting\njp_variable_name_suffix\n```",  # noqa
    ##################################################
    "## Testing": "## テスト",
    ##################################################
    "```{toctree}\n:maxdepth: 1\nassertion_basic_behavior\nassert_equal_and_not_equal\nassert_true_and_false\nassert_greater_and_greater_equal\nassert_less_and_less_equal\nassert_arrays_equal_and_arrays_not_equal\nassert_dicts_equal_and_dicts_not_equal\nassert_defined_and_undefined\n```": "```{toctree}\n:maxdepth: 1\njp_assertion_basic_behavior\njp_assert_equal_and_not_equal\njp_assert_true_and_false\njp_assert_greater_and_greater_equal\njp_assert_less_and_less_equal\njp_assert_arrays_equal_and_arrays_not_equal\njp_assert_dicts_equal_and_dicts_not_equal\njp_assert_defined_and_undefined\n```",  # noqa
}



"""This module is for the translation mapping data of the
following document:

Document file: graphics_draw_dashed_line.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {
    ##################################################
    "# Graphics draw_dashed_line interface": "# Graphics クラスの draw_dashed_line インターフェイス",  # noqa
    ##################################################
    "This page explains the `Graphics` class `draw_dashed_line` method interface.": "このページでは`Graphics`クラスの`draw_dashed_line`メソッドのインターフェイスについて説明します。",  # noqa
    ##################################################
    "## What interface is this?": "## インターフェイス概要",
    ##################################################
    "`draw_dashed_line` interface will draw the simple straight dashed-line graphics. This interface will ignore `dot_setting`, `dash_setting`, `round_dot_setting`, and `dash_dot_setting`.": "`draw_dashed_line`インターフェイスはシンプルな破線の直線のグラフィックスを描画します。このインターフェイスは`dot_setting`や`dash_setting`、`round_dot_setting`、`dash_dot_setting`の引数や属性の設定を無視します。",  # noqa
    ##################################################
    "## Basic usage": "## 基本的な使い方",
    ##################################################
    "`draw_dashed_line` interface has basic coordinates arguments of `x_start`, `y_start`, `x_end` and `y_end`. That also has `dash_size` and `space_size` arguments to determine dash style (line dash size and the space size between each dash).": "`draw_dashed_line`インターフェイスは基本的な線の座標の指定として`x_start`、`y_start`、`x_end`、`y_end`の各引数を持ちます。加えて、破線のサイズとしての`dash_size`引数と破線間のスペースのサイズとしての`space_size`引数の指定が必要です。",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color="#333", stage_width=250, stage_height=130, stage_elem_id="stage"\n)\nsprite: ap.Sprite = ap.Sprite()\n\n# Set 5-pixel dash setting and draw the line.\nsprite.graphics.line_style(color="#0af", thickness=2)\nsprite.graphics.draw_dashed_line(\n    x_start=50, y_start=50, x_end=200, y_end=50, dash_size=5, space_size=2\n)\n\n# Set 10-pixel dash setting and draw the line.\nsprite.graphics.draw_dashed_line(\n    x_start=50, y_start=80, x_end=200, y_end=80, dash_size=10, space_size=2\n)\n\nap.save_overall_html(dest_dir_path="graphics_draw_dashed_line_basic_usage/")\n```': '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color="#333", stage_width=250, stage_height=130, stage_elem_id="stage"\n)\nsprite: ap.Sprite = ap.Sprite()\n\n# Set 5-pixel dash setting and draw the line.\nsprite.graphics.line_style(color="#0af", thickness=2)\nsprite.graphics.draw_dashed_line(\n    x_start=50, y_start=50, x_end=200, y_end=50, dash_size=5, space_size=2\n)\n\n# Set 10-pixel dash setting and draw the line.\nsprite.graphics.draw_dashed_line(\n    x_start=50, y_start=80, x_end=200, y_end=80, dash_size=10, space_size=2\n)\n\nap.save_overall_html(dest_dir_path="graphics_draw_dashed_line_basic_usage/")\n```',  # noqa
    ##################################################
    "## draw_dashed_line API": "## draw_dashed_line API",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "Draw a dashed line vector graphics.<hr>": "破線のベクターグラフィックスを描画します。<hr>",
    ##################################################
    "**[Parameters]**": "**[引数]**",
    ##################################################
    "- `x_start`: Int or int": "- `x_start`: Int or int",
    ##################################################
    "  - Line start x-coordinate.": "  - 線の開始位置のX座標。",
    ##################################################
    "- `y_start`: Int or int": "- `y_start`: Int or int",
    ##################################################
    "  - Line start y-coordinate.": "  - 線の開始位置のY座標。",
    ##################################################
    "- `x_end`: Int or int": "- `x_end`: Int or int",
    ##################################################
    "  - Line end x-coordinate.": "  - 線の終了位置のX座標。",
    ##################################################
    "- `y_end`: Int or int": "- `y_end`: Int or int",
    ##################################################
    "  - Line end y-coordinate.": "  - 線の終了位置のY座標。",
    ##################################################
    "- `dash_size`: Int or int": "- `dash_size`: Int or int",
    ##################################################
    "  - Dash size.": "  - 破線部分のサイズ。",
    ##################################################
    "- `space_size`: Int or int": "- `space_size`: Int or int",
    ##################################################
    "  - Blank space size between dashes.": "  - 破線間の空白スペースのサイズ。",
    ##################################################
    "- `variable_name_suffix`: str, default ''": "- `variable_name_suffix`: str, default ''",  # noqa
    ##################################################
    "  - A JavaScript variable name suffix string. This setting is sometimes useful for JavaScript's debugging.": "  - JavaScript上の変数のサフィックスの設定です。この設定はJavaScriptのデバッグ時に役立つことがあります。",  # noqa
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Returns]**": "**[返却値]**",
    ##################################################
    "- `line`: Line": "- `line`: Line",
    ##################################################
    "  - Created line graphics instance.": "  - 生成された線のグラフィックスのインスタンス。",
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Notes]**": "**[特記事項]**",
    ##################################################
    " ・This interface ignores line settings, like the `LineDotSetting`, except `LineDashSetting`.<hr>": " ・このインターフェイスは`LineDashSetting`を除いた`LineDotSetting`などの線のスタイル設定を無視します。<hr>",  # noqa
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    "```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.line_style(color=\"#fff\", thickness=5)\n>>> line: ap.Line = sprite.graphics.draw_dashed_line(\n...     x_start=50, y_start=50, x_end=150, y_end=50, dash_size=5, space_size=2\n... )\n>>> line.line_color\nString('#ffffff')\n\n>>> line.line_dash_setting.dash_size\nInt(5)\n\n>>> line.line_dash_setting.space_size\nInt(2)\n```": "```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.line_style(color=\"#fff\", thickness=5)\n>>> line: ap.Line = sprite.graphics.draw_dashed_line(\n...     x_start=50, y_start=50, x_end=150, y_end=50, dash_size=5, space_size=2\n... )\n>>> line.line_color\nString('#ffffff')\n\n>>> line.line_dash_setting.dash_size\nInt(5)\n\n>>> line.line_dash_setting.space_size\nInt(2)\n```",  # noqa
}



"""This module is for the translation mapping data of the
following document:

Document file: remove_children.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {
    ##################################################
    "# remove_children interface": "# remove_children インターフェイス",
    ##################################################
    "This page explains the `remove_children` method interface of container classes.": "このページではコンテナの各クラスの`remove_children`メソッドのインターフェイスについて説明します。",  # noqa
    ##################################################
    "## What interface is this?": "## インターフェイス概要",
    ##################################################
    "The `remove_children` method removes all children from a container instance.": "`remove_children`メソッドはコンテナのインスタンスから全ての子のインスタンスを取り除きます。",  # noqa
    ##################################################
    "## Basic usage": "## 基本的な使い方",
    ##################################################
    "The `remove_children` method takes no arguments to use.": "`remove_children`メソッドは引数の指定を必要としません。",  # noqa
    ##################################################
    "In the following example, if you click any rectangle, the handler calls the `remove_children` method and removes all children:": "以下の例では、いずれかの四角をクリックした場合`remove_children`メソッドが呼ばれ全ての子が取り除かれます:",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\n\ndef on_click(e: ap.MouseEvent[ap.Sprite], options: dict) -> None:\n    """\n    The click event handler.\n\n    Parameters\n    ----------\n    e : ap.MouseEvent[ap.Sprite]\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    sprite: ap.Sprite = e.this\n    sprite.remove_children()\n\n\nap.Stage(\n    stage_width=250, stage_height=150, background_color="#333", stage_elem_id="stage"\n)\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color="#0af")\nsprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\nsprite.graphics.draw_rect(x=150, y=50, width=50, height=50)\nsprite.click(on_click)\n\nap.save_overall_html(dest_dir_path="remove_children_basic_usage/")\n```': '```py\n# runnable\nimport apysc as ap\n\n\ndef on_click(e: ap.MouseEvent[ap.Sprite], options: dict) -> None:\n    """\n    The click event handler.\n\n    Parameters\n    ----------\n    e : ap.MouseEvent[ap.Sprite]\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    sprite: ap.Sprite = e.this\n    sprite.remove_children()\n\n\nap.Stage(\n    stage_width=250, stage_height=150, background_color="#333", stage_elem_id="stage"\n)\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color="#0af")\nsprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\nsprite.graphics.draw_rect(x=150, y=50, width=50, height=50)\nsprite.click(on_click)\n\nap.save_overall_html(dest_dir_path="remove_children_basic_usage/")\n```',  # noqa
    ##################################################
    "## remove_children API": "## remove_children のAPI",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "Remove all children from this instance.<hr>": "このインスタンスから全ての子要素を取り除きます。<hr>",
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    '```py\n>>> import apysc as ap\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> rectangle_1: ap.Rectangle = ap.Rectangle(\n...     x=50, y=50, width=50, height=50, fill_color="#0af"\n... )\n>>> rectangle_2: ap.Rectangle = ap.Rectangle(\n...     x=150, y=50, width=50, height=50, fill_color="#0af"\n... )\n>>> sprite.add_child(child=rectangle_1)\n>>> sprite.add_child(child=rectangle_2)\n>>> sprite.remove_children()\n>>> sprite.num_children\nInt(0)\n```': '```py\n>>> import apysc as ap\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> rectangle_1: ap.Rectangle = ap.Rectangle(\n...     x=50, y=50, width=50, height=50, fill_color="#0af"\n... )\n>>> rectangle_2: ap.Rectangle = ap.Rectangle(\n...     x=150, y=50, width=50, height=50, fill_color="#0af"\n... )\n>>> sprite.add_child(child=rectangle_1)\n>>> sprite.add_child(child=rectangle_2)\n>>> sprite.remove_children()\n>>> sprite.num_children\nInt(0)\n```',  # noqa
}



"""This module is for the translation mapping data of the
following document:

Document file: if.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {
    ##################################################
    "# If class": "# If クラス",
    ##################################################
    "This page explains the `If` class.": "このページでは`If`クラスについて説明します。",
    ##################################################
    "Before reading on, maybe it is helpful to read the following page (the apysc uses the `If` class for the same reason of each apysc data type):": "このページを読み進める前に以下のページを確認しておくと役に立つかもしれません（apyscでは基本的なデータクラスと同様の理由で`If`クラスを使用しています）:",  # noqa
    ##################################################
    "- [Why the apysc library doesn't use the Python built-in data type](why_apysc_doesnt_use_python_builtin_data_type.md)": "- [なぜapyscではPythonのビルトインのデータの型を使用していないのか](jp_why_apysc_doesnt_use_python_builtin_data_type.md)",  # noqa
    ##################################################
    "## What is the If class?": "## If クラスの概要",
    ##################################################
    "The `If` class is the apysc branch instruction class. It behaves like the Python built-in `if` keyword.": "`If`クラスはapyscの分岐制御のためのクラスです。このクラスはPythonビルトインの`if`キーワードと似たような形で動作します。",  # noqa
    ##################################################
    "## Basic usage": "## 基本的な使い方",
    ##################################################
    "The `If` class requires the `with` statement as follows:": "`If`クラスは以下のコード例のように`with`ステートメントと共に使用する必要があります:",  # noqa
    ##################################################
    "```py\n# runnable\nimport apysc as ap\n\ncondition: ap.Boolean = ap.Boolean(True)\nwith ap.If(condition):\n    ...\n```": "```py\n# runnable\nimport apysc as ap\n\ncondition: ap.Boolean = ap.Boolean(True)\nwith ap.If(condition):\n    ...\n```",  # noqa
    ##################################################
    "The `If` class requires passing the `Boolean` value as the condition.": "`If`クラスのコンストラクタの引数には条件としての`Boolean`の値の指定が必要になります。",  # noqa
    ##################################################
    "## See also": "## 関連資料",
    ##################################################
    "- [Elif class](elif.md)": "- [Elif クラス](jp_elif.md)",
    ##################################################
    "- [Else class](else.md)": "- [Else クラス](jp_else.md)",
    ##################################################
    "- [Each branch instruction class's scope variables reverting setting](branch_instruction_variables_reverting_setting.md)": "- [分岐条件の各クラスのスコープ内変数の復元設定](jp_branch_instruction_variables_reverting_setting.md)",  # noqa
    ##################################################
    "## If constructor API": "## If クラスのコンストラクタのAPI",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "A class to append if branch instruction expression.<hr>": "if文の分岐制御の表現を追加するためのクラス。<hr>",  # noqa
    ##################################################
    "**[Parameters]**": "**[引数]**",
    ##################################################
    "- `condition`: Boolean or None": "- `condition`: Boolean or None",
    ##################################################
    "  - Boolean value to be used for judgment.": "  - 判定に使われるBooleanの真偽値。",
    ##################################################
    "- `locals_`: dict or None, default None": "- `locals_`: dict or None, default None",  # noqa
    ##################################################
    "  - Current scope's local variables. Set locals() value to this argument. If specified, this interface reverts all local scope VariableNameMixIn variables (like Int, Sprite) at the end of an `If` scope. This setting is useful when you don't want to update each variable by implementing the `If` scope.": "  - 現在のスコープの各ローカル変数。指定する場合にはlocals()関数の値をごの引数に指定してください。もし指定された場合、このインターフェイスは`If`のスコープの終了時に対象のVariableNameMixInクラスの各ローカル変数のインスタンスの値をスコープの開始前の時点に復元します。この設定は`If`のスコープ内の処理でPython上の各ローカル変数の値を更新したくない場合などに便利なことがあります。",  # noqa
    ##################################################
    "- `globals_`: dict or None, default None": "- `globals_`: dict or None, default None",  # noqa
    ##################################################
    "  - Current scope's global variables. Set globals() value to this argument. This setting works the same way as the locals_ argument.": "  - 現在のスコープの各グローバル変数。設定する場合にはglobal()関数の値をこの引数に指定してください。この設定はlocals_引数と同じように動作します。",  # noqa
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    '```py\n>>> import apysc as ap\n>>> int_val: ap.Int = ap.Int(10)\n>>> condition: ap.Boolean = int_val >= 10\n>>> with ap.If(condition):\n...     ap.trace("Int value is greater than equal 10!")\n...\n```': '```py\n>>> import apysc as ap\n>>> int_val: ap.Int = ap.Int(10)\n>>> condition: ap.Boolean = int_val >= 10\n>>> with ap.If(condition):\n...     ap.trace("Int value is greater than equal 10!")\n...\n```',  # noqa
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[References]**": "**[関連資料]**",
    ##################################################
    "- [Each branch instruction class's scope variables reverting setting](https://simon-ritchie.github.io/apysc/en/branch_instruction_variables_reverting_setting.html)": "- [分岐条件の各クラスのスコープ内変数の復元設定](https://simon-ritchie.github.io/apysc/jp/jp_branch_instruction_variables_reverting_setting.html)",  # noqa
}



"""This module is for the translation mapping data of the
following document:

Document file: get_child_at.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {
    ##################################################
    "# get_child_at interface": "# get_child_at インターフェイス",
    ##################################################
    "This page explains the container class, like the `Graphics`\\, `Sprite`\\, `Stage`) `get_child_at` method interface.": "このページでは`Graphisc`や`Sprite`、`Stage`などのコンテナーのクラスの`get_child_at`メソッドのインターフェイスについて説明します。",  # noqa
    ##################################################
    "## What interface is this?": "## インターフェイス概要",
    ##################################################
    "The `get_child_at` interface returns a child (`DisplayObject`) instance at the specified index.": "`get_child_at`インターフェイスは指定されたインデックスの位置の子のインスタンス（`DisplkayObject`クラスのインスタンス）を返却します。",  # noqa
    ##################################################
    "## Basic usage": "## 基本的な使い方",
    ##################################################
    "The following code example is adding the rectangle to the sprite container. The `Sprite` class adds the `Graphics` instance at the constructor so that the first child becomes the `Graphics` instance. The second child becomes the `Rectangle` instance, which the sprite added with the `add_child` method.": "以下のコード例ではSpriteのコンテナーへと四角を追加しています。`Sprite`のクラスは`Graphics`のインスタンスを子としてコンストラクタで追加するため、最初の子は`Graphics`のインスタンスとなり、2番目の子は`add_child`メソッドで追加された`Rectangle`の四角のインスタンスとなります。",  # noqa
    ##################################################
    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color="#333", stage_width=450, stage_height=150, stage_elem_id="stage"\n)\n\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color="#0af")\nrectangle_1: ap.Rectangle = sprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\nsprite.add_child(rectangle_1)\n\nfirst_child: ap.DisplayObject = sprite.get_child_at(index=0)\nassert isinstance(first_child, ap.Graphics)\n\nsecond_child: ap.DisplayObject = sprite.get_child_at(index=1)\nassert isinstance(second_child, ap.Rectangle)\n```': '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color="#333", stage_width=450, stage_height=150, stage_elem_id="stage"\n)\n\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color="#0af")\nrectangle_1: ap.Rectangle = sprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\nsprite.add_child(rectangle_1)\n\nfirst_child: ap.DisplayObject = sprite.get_child_at(index=0)\nassert isinstance(first_child, ap.Graphics)\n\nsecond_child: ap.DisplayObject = sprite.get_child_at(index=1)\nassert isinstance(second_child, ap.Rectangle)\n```',  # noqa
    ##################################################
    "## get_child_at API": "## get_child_at API",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "Get a child at a specified index.<hr>": "指定されたインデックスの子を取得します。<hr>",
    ##################################################
    "**[Parameters]**": "**[引数]**",
    ##################################################
    "- `index`: int or Int": "- `index`: int or Int",
    ##################################################
    "  - Child's index (start from 0).": "  - 対象の子のインデックス（0からスタートします）。",
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Returns]**": "**[返却値]**",
    ##################################################
    "- `child`: DisplayObject": "- `child`: DisplayObject",
    ##################################################
    "  - Target index child instance.": "  - 対象の子のインスタンス。",
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    '```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color="#0af", alpha=0.5)\n>>> rectangle_1: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50\n... )\n>>> rectangle_2: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=150, y=50, width=50, height=50\n... )\n>>> child_at_index_1: ap.DisplayObject = sprite.graphics.get_child_at(1)\n>>> child_at_index_1 == rectangle_2\nTrue\n```': '```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color="#0af", alpha=0.5)\n>>> rectangle_1: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50\n... )\n>>> rectangle_2: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=150, y=50, width=50, height=50\n... )\n>>> child_at_index_1: ap.DisplayObject = sprite.graphics.get_child_at(1)\n>>> child_at_index_1 == rectangle_2\nTrue\n```',  # noqa
}



"""This module is for the translation mapping data of the
following document:

Document file: timer.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {
    ##################################################
    "# Timer class": "# Timer クラス",
    ##################################################
    "This page explains the `Timer` class.": "このページでは`Timer`クラスについて説明します。",
    ##################################################
    "## What is the Timer?": "## Timer クラスの概要",
    ##################################################
    "The `Timer` class handles the timer's tick. You can call a handler at any intervals by it.": "`Timer`クラスは一定間隔で処理を実行するためのタイマーの処理を扱います。任意の間隔を設定してハンドラの呼び出し設定を追加することができます。",  # noqa
    ##################################################
    "## Basic usage": "## 基本的な使い方",
    ##################################################
    "The `Timer` class requires arguments for the `handler` and `delay` (timer interval in milliseconds). And the `start` method starts that timer. A timer passes the `TimerEvent` instance and options arguments to a specified handler.": "`Timer`クラスはコンストラクタでハンドラとしての`handler`引数とタイマー実行間隔のミリ秒としての`delay`引数の指定を必要とします。そして`start`メソッドを呼び出すとタイマーがスタートします。タイマーは`TimerEvent`クラスのインスタンスとオプションとして指定できる追加のパラメーターの引数をハンドラへ渡します。",  # noqa
    ##################################################
    "The following code sets the `Timer` when clicking the rectangle (`Sprite`):": "以下のコード例では四角（`Sprite`）をクリックした際に`Timer`クラスを使用した設定を行っています:",  # noqa
    ##################################################
    '```py\n# runnable\nfrom typing_extensions import TypedDict\n\nimport apysc as ap\n\n\nclass _RectOptions(TypedDict):\n    rectangle: ap.Rectangle\n\n\ndef on_sprite_click(e: ap.MouseEvent[ap.Sprite], options: _RectOptions) -> None:\n    """\n    The Handler that the rectangle calls when clicked.\n\n    Parameters\n    ----------\n    e : MouseEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    e.this.unbind_click_all()\n    timer: ap.Timer = ap.Timer(on_timer, delay=16.6, options=options)\n    timer.start()\n\n\ndef on_timer(e: ap.TimerEvent, options: _RectOptions) -> None:\n    """\n    The Handler a timer calls.\n\n    Parameters\n    ----------\n    e : TimerEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    rectangle: ap.Rectangle = options["rectangle"]\n    rectangle.x += 1\n\n\nap.Stage(\n    stage_width=350, stage_height=150, background_color="#333", stage_elem_id="stage"\n)\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color="#0af")\nrectangle: ap.Rectangle = sprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\noptions: _RectOptions = {"rectangle": rectangle}\nsprite.click(on_sprite_click, options=options)\n\nap.save_overall_html(dest_dir_path="timer_basic_usage/")\n```': '```py\n# runnable\nfrom typing_extensions import TypedDict\n\nimport apysc as ap\n\n\nclass _RectOptions(TypedDict):\n    rectangle: ap.Rectangle\n\n\ndef on_sprite_click(e: ap.MouseEvent[ap.Sprite], options: _RectOptions) -> None:\n    """\n    The Handler that the rectangle calls when clicked.\n\n    Parameters\n    ----------\n    e : MouseEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    e.this.unbind_click_all()\n    timer: ap.Timer = ap.Timer(on_timer, delay=16.6, options=options)\n    timer.start()\n\n\ndef on_timer(e: ap.TimerEvent, options: _RectOptions) -> None:\n    """\n    The Handler a timer calls.\n\n    Parameters\n    ----------\n    e : TimerEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    rectangle: ap.Rectangle = options["rectangle"]\n    rectangle.x += 1\n\n\nap.Stage(\n    stage_width=350, stage_height=150, background_color="#333", stage_elem_id="stage"\n)\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color="#0af")\nrectangle: ap.Rectangle = sprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\noptions: _RectOptions = {"rectangle": rectangle}\nsprite.click(on_sprite_click, options=options)\n\nap.save_overall_html(dest_dir_path="timer_basic_usage/")\n```',  # noqa
    ##################################################
    "If you click the rectangle, the timer starts, and the Handler increases the rectangle x value.": "四角をクリックするとタイマーがスタートし、タイマーのハンドラは四角のX座標を加算していきます。",  # noqa
    ##################################################
    "## See also": "## 関連資料",
    ##################################################
    "- [TimerEvent class](timer_event.md)": "- [TimerEvent クラス](jp_timer_event.md)",
    ##################################################
    "- [Timer class delay setting](timer_delay.md)": "- [Timer クラスの delay 設定](jp_timer_delay.md)",  # noqa
    ##################################################
    "- [FPS enum](fps.md)": "- [FPS の enum](jp_fps.md)",
    ##################################################
    "- [Timer class repeat_count setting](timer_repeat_count.md)": "- [Timer クラスの repeat_count 設定](jp_timer_repeat_count.md)",  # noqa
    ##################################################
    "- [Timer class start and stop interfaces](timer_start_and_stop.md)": "- [Timer クラスの start と stop の各インターフェイス](jp_timer_start_and_stop.md)",  # noqa
    ##################################################
    "- [Timer class timer_complete interface](timer_complete.md)": "- [Timer クラスの timer_complete インターフェイス](jp_timer_complete.md)",  # noqa
    ##################################################
    "- [Timer class reset interface](timer_reset.md)": "- [Timer クラスの reset インターフェイス](jp_timer_reset.md)",  # noqa
    ##################################################
    "## Timer constructor API": "## Timer クラスのコンストラクタのAPI",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "Timer class to handle function calling at regular intervals.<hr>": "一定間隔ごとにハンドラの関数を実行するためのタイマーのクラスです。<hr>",  # noqa
    ##################################################
    "**[Parameters]**": "**[引数]**",
    ##################################################
    "- `handler`: _Handler": "- `handler`: _Handler",
    ##################################################
    "  - A handler would be called at regular intervals.": "  - 一定間隔ごとに呼ばれる関数もしくはメソッドのハンドラ。",  # noqa
    ##################################################
    "- `delay`: Int or int or Number or float or FPS": "- `delay`: Int or int or Number or float or FPS",  # noqa
    ##################################################
    "  - A delay between each `Handler` calling in a millisecond or FPS value. If an `FPS` value is specified, this value becomes a millisecond calculated with that FPS value (e.g., if the `FPS_60` value is specified, then `delay` becomes 16.6666667).": "  - ハンドラの実行間隔となるミリ秒もしくはFPSのenumの値。もし`FPS`の値が指定された場合、FPSに応じて計算されたミリ秒が設定されます（例えば、もし`FPS_60`が指定されていれば`delay`の値は16.6666667ミリ秒相当になります。）。",  # noqa
    ##################################################
    "- `repeat_count`: Int or int": "- `repeat_count`: Int or int",
    ##################################################
    "  - Max count of a `Handler`'s calling. A timer stops if the `Handler`'s calling count has reached this value. If 0 is specified, then a timer loops forever.": "  - ハンドラの実行回数の上限値。ハンドラの実行回数がこの値に到達した場合タイマーは停止します。もし0が指定された場合にはタイマーは停止しなくなります。",  # noqa
    ##################################################
    "- `options`: dict or None, default None": "- `options`: dict or None, default None",  # noqa
    ##################################################
    "  - Optional arguments dictionary to pass a `Handler` callable.": "  - ハンドラの関数もしくはメソッドへ渡すオプションとしての各パラメーターを格納した辞書。",  # noqa
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    '```py\n>>> from typing_extensions import TypedDict\n>>> import apysc as ap\n>>> class RectOptions(TypedDict):\n...     rectangle: ap.Rectangle\n...\n>>> def on_timer(e: ap.TimerEvent, options: RectOptions) -> None:\n...     rectangle: ap.Rectangle = options["rectangle"]\n...     rectangle.x += 1\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color="#0af")\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50\n... )\n>>> options: RectOptions = {"rectangle": rectangle}\n>>> _ = ap.Timer(on_timer, delay=ap.FPS.FPS_60, options=options).start()\n```': '```py\n>>> from typing_extensions import TypedDict\n>>> import apysc as ap\n>>> class RectOptions(TypedDict):\n...     rectangle: ap.Rectangle\n...\n>>> def on_timer(e: ap.TimerEvent, options: RectOptions) -> None:\n...     rectangle: ap.Rectangle = options["rectangle"]\n...     rectangle.x += 1\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color="#0af")\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50\n... )\n>>> options: RectOptions = {"rectangle": rectangle}\n>>> _ = ap.Timer(on_timer, delay=ap.FPS.FPS_60, options=options).start()\n```',  # noqa
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[References]**": "**[関連資料]**",
    ##################################################
    "- [TimerEvent class](https://simon-ritchie.github.io/apysc/en/timer_event.html)": "- [TimerEvent クラス](https://simon-ritchie.github.io/apysc/jp/jp_timer_event.html)",  # noqa
    ##################################################
    "- [Timer class delay setting](https://simon-ritchie.github.io/apysc/en/timer_delay.html)": "- [Timer クラスの delay 設定](https://simon-ritchie.github.io/apysc/jp/jp_timer_delay.html)",  # noqa
    ##################################################
    "- [FPS enum](https://simon-ritchie.github.io/apysc/en/fps.html)": "- [FPS の enum](https://simon-ritchie.github.io/apysc/jp/jp_fps.html)",  # noqa
    ##################################################
    "- [Timer class repeat_count setting](https://simon-ritchie.github.io/apysc/en/timer_repeat_count.html)": "- [Timer クラスの repeat_count 設定](https://simon-ritchie.github.io/apysc/jp/jp_timer_repeat_count.html)",  # noqa
    ##################################################
    "- [About the handler options' type](https://simon-ritchie.github.io/apysc/en/about_handler_options_type.html)": "- [ハンドラのoptions引数の型について](https://simon-ritchie.github.io/apysc/jp/jp_about_handler_options_type.html)",  # noqa
}



"""This module is for the translation mapping data of the
following document:

Document file: array_length.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {
    ##################################################
    "# Array class length interface": "# Array クラスの length インターフェイス",
    ##################################################
    "This page explains the `Array` class `length` property interface.": "このページでは`Array`クラスの`length`属性のインターフェイスについて説明します。",  # noqa
    ##################################################
    "## What interface is this?": "## インターフェイス概要",
    ##################################################
    "The `length` property interface returns a current array's values length.": "`length`属性のインターフェイスは配列の値の長さ（件数）を返却します。",  # noqa
    ##################################################
    "## Basic usage": "## 基本的な使い方",
    ##################################################
    "The `length` property has the only getter interface. The return value type is the `Int` type.": "`length`属性はgetterのインターフェイスのみを持ちます。返却値はapyscの`Int`クラスの整数値となります。",  # noqa
    ##################################################
    "```py\n# runnable\nimport apysc as ap\n\narr: ap.Array[int] = ap.Array([1, 2, 3, 4])\nlength: ap.Int = arr.length\nassert length == 4\n```": "```py\n# runnable\nimport apysc as ap\n\narr: ap.Array[int] = ap.Array([1, 2, 3, 4])\nlength: ap.Int = arr.length\nassert length == 4\n```",  # noqa
    ##################################################
    "## Notes of the len() function": "## len()関数における特記事項",
    ##################################################
    "The `Array` class is not supported the Python built-in `len()` function, and its function raises an exception. Please use the `length` property instead.": "`Array`クラスはPythonビルトインの`len()`関数をサポートしておらず、もし使用した場合にはエラーとなります。代わりに`length`属性を使用してください。",  # noqa
    ##################################################
    "```py\nimport apysc as ap\n\narr: ap.Array[int] = ap.Array([1, 2, 3, 4])\nlen(arr)\n```": "```py\nimport apysc as ap\n\narr: ap.Array[int] = ap.Array([1, 2, 3, 4])\nlen(arr)\n```",  # noqa
    ##################################################
    "```\nException: Array instance can't apply len function. Please use length property instead.\n```": "```\nException: Array instance can't apply len function. Please use length property instead.\n```",  # noqa
    ##################################################
    "## length property API": "## length属性のAPI",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "Get length of this array.<hr>": "この配列の長さ（値の数）を取得します。<hr>",
    ##################################################
    "**[Returns]**": "**[返却値]**",
    ##################################################
    "- `length`: Int": "- `length`: Int",
    ##################################################
    "  - This array's length.": "  - この配列の長さ（値の件数）。",
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    "```py\n>>> import apysc as ap\n>>> arr: ap.Array = ap.Array([1, 2, 3])\n>>> arr.length\nInt(3)\n```": "```py\n>>> import apysc as ap\n>>> arr: ap.Array = ap.Array([1, 2, 3])\n>>> arr.length\nInt(3)\n```",  # noqa
}



"""Class implementation for boolean.
"""

from typing import Any
from typing import Dict
from typing import Union

from typing_extensions import final

from apysc._event.custom_event_mixin import CustomEventMixIn
from apysc._html.debug_mode import add_debug_info_setting
from apysc._type.copy_mixin import CopyMixIn
from apysc._type.initial_substitution_exp_mixin import InitialSubstitutionExpMixIn
from apysc._type.int import Int
from apysc._type.revert_mixin import RevertMixIn
from apysc._type.variable_name_mixin import VariableNameMixIn
from apysc._type.variable_name_suffix_mixin import VariableNameSuffixMixIn
from apysc._validation import arg_validation_decos


class Boolean(
    CopyMixIn,
    RevertMixIn,
    CustomEventMixIn,
    VariableNameSuffixMixIn,
    InitialSubstitutionExpMixIn,
):
    """
    Boolean class for the apysc library.

    Notes
    -----
    The Bool class is the alias of the Boolean, and it behaves
    the same as the Boolean class.

    References
    ----------
    - Boolean
        - https://simon-ritchie.github.io/apysc/en/boolean.html

    Examples
    --------
    >>> import apysc as ap
    >>> bool_val_1: ap.Boolean = ap.Boolean(True)
    >>> bool_val_1
    Boolean(True)

    >>> bool_val_2: ap.Bool = ap.Bool(True)
    >>> bool_val_2
    Boolean(True)

    >>> bool_val_2.not_
    Boolean(False)
    """

    _initial_value: Union[bool, int, Int, "Boolean"]
    _value: bool

    @arg_validation_decos.is_acceptable_boolean_value(arg_position_index=1)
    @arg_validation_decos.is_builtin_string(arg_position_index=2, optional=False)
    @add_debug_info_setting(module_name=__name__)
    def __init__(
        self,
        value: Union[bool, int, Int, "Boolean"],
        *,
        variable_name_suffix: str = "",
        skip_init_substitution_expression_appending: bool = False,
    ) -> None:
        """
        Boolean class for apysc library.

        Notes
        -----
        The Bool class is the alias of the Boolean, and it behaves
        the same as the Boolean class.

        Parameters
        ----------
        value : Boolean or Int or bool or int
            Initial boolean value. 0 or 1 are acceptable for
            an integer value.
        variable_name_suffix : str, default ''
            A JavaScript variable name suffix string.
            This setting is sometimes useful for JavaScript's debugging.
        skip_init_substitution_expression_appending : bool, default False
            A boolean indicates whether to skip an initial substitution
            expression or not. This class uses this option internally.

        References
        ----------
        - Boolean
            - https://simon-ritchie.github.io/apysc/en/boolean.html

        Examples
        --------
        >>> import apysc as ap
        >>> bool_val_1: ap.Boolean = ap.Boolean(True)
        >>> bool_val_1
        Boolean(True)

        >>> bool_val_2: ap.Bool = ap.Bool(True)
        >>> bool_val_2
        Boolean(True)
        """
        from apysc._expression import expression_variables_util
        from apysc._expression import var_names
        from apysc._expression.event_handler_scope import TemporaryNotHandlerScope

        with TemporaryNotHandlerScope():
            self._variable_name_suffix = variable_name_suffix
            TYPE_NAME: str = var_names.BOOLEAN
            self._initial_value = value
            value_: bool = self._get_bool_from_arg_value(value=value)
            self._value = value_
            self._type_name = TYPE_NAME
            self.variable_name = expression_variables_util.get_next_variable_name(
                type_name=TYPE_NAME
            )
            self._append_constructor_expression()

        self._append_initial_substitution_expression_if_in_handler_scope(
            skip_appending=skip_init_substitution_expression_appending,
        )

    @final
    @add_debug_info_setting(module_name=__name__)
    def _get_bool_from_arg_value(
        self, *, value: Union[bool, int, Int, "Boolean"]
    ) -> bool:
        """
        Get bool value from specified argument value.

        Parameters
        ----------
        value : Boolean or Int or bool or int
            Specified value. 0 or 1 are acceptable for
            an integer value.

        Returns
        -------
        result : bool
            Converted boolean value.
        """
        from apysc._converter import cast
        from apysc._type.number_value_mixin import NumberValueMixIn
        from apysc._validation import bool_validation

        if isinstance(value, (int, float, NumberValueMixIn)):
            result: bool = cast.to_bool_from_int(integer=value)  # type: ignore
        elif isinstance(value, Boolean):
            result = value._value
        else:
            result = value  # type: ignore
        bool_validation.validate_bool(value=result)
        return result

    @final
    @add_debug_info_setting(module_name=__name__)
    def _append_constructor_expression(self) -> None:
        """
        Append constructor expression.
        """
        import apysc as ap

        expression: str = self._create_initial_substitution_expression()
        expression = f"var {expression}"
        ap.append_js_expression(expression=expression)

    @final
    def _create_initial_substitution_expression(self) -> str:
        """
        Create an initial value's substitution expression string.

        Returns
        -------
        expression : str
            Created expression string.
        """
        expression: str = f"{self.variable_name} = "
        if isinstance(self._initial_value, VariableNameMixIn):
            expression += f"Boolean({self._initial_value.variable_name});"
        elif self._value:
            expression += "true;"
        else:
            expression += "false;"
        return expression

    @property
    @add_debug_info_setting(module_name=__name__)
    def value(self) -> Union[bool, int, Int, "Boolean"]:
        """
        Get a current boolean value.

        Returns
        -------
        value : bool
            Current boolean value.

        References
        ----------
        - apysc fundamental data classes value interface
            - https://simon-ritchie.github.io/apysc/en/fundamental_data_classes_value_interface.html  # noqa

        Examples
        --------
        >>> import apysc as ap
        >>> bool_val: ap.Boolean = ap.Boolean(True)
        >>> bool_val.value = False
        >>> bool_val.value
        False

        >>> bool_val.value = ap.Boolean(True)
        >>> bool_val.value
        True
        """
        return self._value

    @value.setter
    @arg_validation_decos.is_acceptable_boolean_value(arg_position_index=1)
    @add_debug_info_setting(module_name=__name__)
    def value(self, value: Union[bool, int, Int, "Boolean"]) -> None:
        """
        Set boolean value.

        Parameters
        ----------
        value : Boolean or Int or bool or int
            Any boolean value to set.

        References
        ----------
        - apysc fundamental data classes value interface
            - https://simon-ritchie.github.io/apysc/en/fundamental_data_classes_value_interface.html  # noqa
        """
        from apysc._type.variable_name_mixin import VariableNameMixIn

        self._set_value_and_skip_expression_appending(value=value)
        if isinstance(value, VariableNameMixIn):
            self._append_value_setter_expression(value=value)
        else:
            self._append_value_setter_expression(value=self._value)

    @final
    @add_debug_info_setting(module_name=__name__)
    def _append_value_setter_expression(
        self, *, value: Union[bool, int, Int, "Boolean"]
    ) -> None:
        """
        Append value's setter expression.

        Parameters
        ----------
        value : bool or VariableNameMixIn
            Any value to set.
        """
        import apysc as ap
        from apysc._type.variable_name_mixin import VariableNameMixIn

        expression: str = f"{self.variable_name} = "
        if isinstance(value, VariableNameMixIn):
            expression += f"Boolean({value.variable_name});"
        elif value:
            expression += "true;"
        else:
            expression += "false;"
        ap.append_js_expression(expression=expression)

    @final
    def _set_value_and_skip_expression_appending(
        self, *, value: Union[bool, int, Int, "Boolean"]
    ) -> None:
        """
        Update value attribute and skip expression appending.

        Parameters
        ----------
        value : Boolean or Int or bool or int
            Any boolean value to set.
        """
        value_: bool = self._get_bool_from_arg_value(value=value)
        self._value = value_

    @final
    def __bool__(self) -> bool:
        """
        Get a boolean value directly.

        Returns
        -------
        result : bool
            Current boolean value.
        """
        return self._value

    def __repr__(self) -> str:
        """
        Get a representation string of this instance.

        Returns
        -------
        repr_str : str
            Representation string of this instance.
        """
        if not hasattr(self, "_value"):
            repr_str: str = "Boolean(False)"
        else:
            repr_str = f"Boolean({self._value})"
        return repr_str

    _value_snapshots: Dict[str, bool]

    def _make_snapshot(self, *, snapshot_name: str) -> None:
        """
        Make value's snapshot.

        Parameters
        ----------
        snapshot_name : str
            Target snapshot name.
        """
        self._set_single_snapshot_val_to_dict(
            dict_name="_value_snapshots", value=self._value, snapshot_name=snapshot_name
        )

    def _revert(self, *, snapshot_name: str) -> None:
        """
        Revert value if snapshot exists.

        Parameters
        ----------
        snapshot_name : str
            Target snapshot name.
        """
        if not self._snapshot_exists(snapshot_name=snapshot_name):
            return
        self._value = self._value_snapshots[snapshot_name]

    @final
    @add_debug_info_setting(module_name=__name__)
    def __eq__(self, other: Any) -> Any:
        """
        Comparison method for equal condition.

        Parameters
        ----------
        other : *
            The other value to compare.

        Returns
        -------
        result : Boolean
            Comparison result.
        """
        import apysc as ap

        self._validate_comparison_other_type(other=other)
        result: Boolean
        if isinstance(other, Boolean):
            result = Boolean(
                self._value == other._value,
                variable_name_suffix=self._variable_name_suffix,
            )
            self._append_eq_expression(result=result, other=other)
            return result
        elif isinstance(other, ap.Int):
            other_ = bool(other.value)
            result = Boolean(
                self._value == other_, variable_name_suffix=self._variable_name_suffix
            )
        else:
            other = bool(other)
            result = Boolean(
                self._value == other, variable_name_suffix=self._variable_name_suffix
            )
            other = Boolean(other, variable_name_suffix=self._variable_name_suffix)
        self._append_eq_expression(result=result, other=other)
        return result

    @final
    def _validate_comparison_other_type(self, *, other: Any) -> None:
        """
        Validate a comparison's other value type.

        Parameters
        ----------
        other : *
            The other value to compare.

        Raises
        ------
        ValueError
            If the other value type is not Boolean, Int, bool,
            and int.
        """
        import apysc as ap

        ACCEPTABLE_TYPES: tuple = (Boolean, bool, ap.Int, int)
        if isinstance(other, ACCEPTABLE_TYPES):
            return
        raise ValueError(
            "Can't acceptable comparison value type is specified: "
            f"{type(other)}, {other}"
            f"\nAcceptable value types are: {ACCEPTABLE_TYPES}"
        )

    @final
    @add_debug_info_setting(module_name=__name__)
    def _append_eq_expression(
        self, *, result: VariableNameMixIn, other: VariableNameMixIn
    ) -> None:
        """
        Append __eq__ method expression.

        Parameters
        ----------
        result : Boolean
            A result boolean value.
        other : Boolean or Int
            The other value to compare.
        """
        import apysc as ap

        other_str: str = other.variable_name
        if isinstance(other, ap.Int):
            other_str = f"Boolean({other_str});"
        expression: str = (
            f"{result.variable_name} = " f"{self.variable_name} === {other_str};"
        )
        ap.append_js_expression(expression=expression)

    @final
    @add_debug_info_setting(module_name=__name__)
    def __ne__(self, other: Any) -> Any:
        """
        Comparison method for not equal condition.

        Parameters
        ----------
        other : *
            The other value to compare.

        Returns
        -------
        result : Boolean
            Comparison result.
        """
        import apysc as ap

        result: Boolean = self == other
        result = result.not_
        if isinstance(other, (Boolean, ap.Int)):
            self._append_ne_expression(result=result, other=other)
        return result

    @final
    @add_debug_info_setting(module_name=__name__)
    def _append_ne_expression(
        self, *, result: VariableNameMixIn, other: VariableNameMixIn
    ) -> None:
        """
        Append __ne__ method expression.

        Parameters
        ----------
        result : Boolean
            A result boolean value.
        other : Boolean or Int
            The other value to compare.
        """
        import apysc as ap

        other_str: str = other.variable_name
        if isinstance(other, ap.Int):
            other_str = f"Boolean({other_str});"
        expression: str = (
            f"{result.variable_name} = " f"{self.variable_name} !== {other_str};"
        )
        ap.append_js_expression(expression=expression)

    @property
    @add_debug_info_setting(module_name=__name__)
    def not_(self) -> "Boolean":
        """
        Get a not condition Boolean value.

        Returns
        -------
        result : Boolean
            Not condition Boolean value.

        Examples
        --------
        >>> import apysc as ap
        >>> bool_val: ap.Boolean = ap.Boolean(True)
        >>> bool_val.not_
        Boolean(False)

        >>> bool_val.value = False
        >>> bool_val.not_
        Boolean(True)
        """
        result: Boolean = Boolean(
            not self, variable_name_suffix=self._variable_name_suffix
        )
        self._append_not_prop_expression(result=result)
        return result

    @final
    @add_debug_info_setting(module_name=__name__)
    def _append_not_prop_expression(self, *, result: VariableNameMixIn) -> None:
        """
        Append not_ property expression.

        Parameters
        ----------
        result : Boolean
            A result Boolean value.
        """
        import apysc as ap

        expression: str = f"{result.variable_name} = " f"!{self.variable_name};"
        ap.append_js_expression(expression=expression)



"""Class implementations for datetime-related interfaces.
"""

from datetime import datetime
from typing import Union

from typing_extensions import final

from apysc._html.debug_mode import add_debug_info_setting
from apysc._time.day_mixin import DayMixIn
from apysc._time.hour_mixin import HourMixIn
from apysc._time.millisecond_mixin import MillisecondMixIn
from apysc._time.minute_mixin import MinuteMixIn
from apysc._time.month_end_mixin import MonthEndMixin
from apysc._time.month_mixin import MonthMixIn
from apysc._time.second_mixin import SecondMixIn
from apysc._time.weekday_mixin import WeekdayMixIn
from apysc._time.year_mixin import YearMixIn
from apysc._type.initial_substitution_exp_mixin import InitialSubstitutionExpMixIn
from apysc._type.int import Int
from apysc._type.variable_name_suffix_mixin import VariableNameSuffixMixIn
from apysc._validation import arg_validation_decos


class DateTime(
    VariableNameSuffixMixIn,
    InitialSubstitutionExpMixIn,
    YearMixIn,
    MonthMixIn,
    DayMixIn,
    HourMixIn,
    MinuteMixIn,
    SecondMixIn,
    MillisecondMixIn,
    WeekdayMixIn,
    MonthEndMixin,
):
    """
    The class for datetime-related interfaces.

    Attributes
    ----------
    year : Int
        A current year value.
    month : Int
        A current month value.
    day : Int
        A current day value.
    hour : Int
        A current hour value.
    minute : Int
        A current minute value.
    second : Int
        A current second value.
    millisecond : Int
        A current millisecond value.
    weekday_js : Int
        A current JavaScript's weekday value (Sunday is 0).
    weekday_py : int
        A current Python's weekday value (Monday is 0).

    References
    ----------
    - DateTime class
        - https://simon-ritchie.github.io/apysc/en/datetime.html
    - DateTime class year property
        - https://simon-ritchie.github.io/apysc/en/datetime_year.html
    - DateTime class month property
        - https://simon-ritchie.github.io/apysc/en/datetime_month.html
    - DateTime class day property
        - https://simon-ritchie.github.io/apysc/en/datetime_day.html
    - DateTime class hour property
        - https://simon-ritchie.github.io/apysc/en/datetime_hour.html
    - DateTime class minute property
        - https://simon-ritchie.github.io/apysc/en/datetime_minute.html
    - DateTime class second property
        - https://simon-ritchie.github.io/apysc/en/datetime_second.html
    - DateTime class millisecond property
        - https://simon-ritchie.github.io/apysc/en/datetime_millisecond.html
    - DateTime class weekday_js and weekday_py properties
        - https://simon-ritchie.github.io/apysc/en/datetime_weekday_js_and_weekday_py.html  # noqa
    - DateTime class now interface
        - https://simon-ritchie.github.io/apysc/en/datetime_now.html

    Examples
    --------
    >>> import apysc as ap
    >>> datetime_: ap.DateTime = ap.DateTime(
    ...     year=2022,
    ...     month=12,
    ...     day=5,
    ...     hour=10,
    ...     minute=30,
    ...     second=50,
    ...     millisecond=500,
    ... )
    >>> datetime_.year
    Int(2022)
    >>> datetime_.month
    Int(12)
    >>> datetime_.day
    Int(5)
    >>> datetime_.hour
    Int(10)
    >>> datetime_.minute
    Int(30)
    >>> datetime_.millisecond
    Int(500)
    >>> datetime_.weekday_py
    Int(0)
    >>> datetime_.weekday_js
    Int(1)
    """

    @arg_validation_decos.is_four_digit_year(arg_position_index=1)
    @arg_validation_decos.is_month_int(arg_position_index=2)
    @arg_validation_decos.is_day_int(arg_position_index=3)
    @arg_validation_decos.is_hour_int(arg_position_index=4)
    @arg_validation_decos.is_minute_int(arg_position_index=5)
    @arg_validation_decos.is_second_int(arg_position_index=6)
    @arg_validation_decos.is_millisecond_int(arg_position_index=7)
    @arg_validation_decos.is_builtin_string(arg_position_index=8, optional=False)
    @arg_validation_decos.is_builtin_boolean(arg_position_index=9)
    @add_debug_info_setting(module_name=__name__)
    def __init__(
        self,
        year: Union[int, Int],
        month: Union[int, Int],
        day: Union[int, Int],
        *,
        hour: Union[int, Int] = 0,
        minute: Union[int, Int] = 0,
        second: Union[int, Int] = 0,
        millisecond: Union[int, Int] = 0,
        variable_name_suffix: str = "",
        skip_init_substitution_expression_appending: bool = False,
    ) -> None:
        """
        The class for datetime-related interfaces.

        Parameters
        ----------
        year : Union[int, Int]
            Four-digit year.
        month : Union[int, Int]
            Two-digit month (1 to 12).
        day : Union[int, Int]
            Two-digit day (1 to 31).
        hour : Optional[Union[int, Int]], optional
            Two-digit hour (0 to 23).
        minute : Optional[Union[int, Int]], optional
            Two-digit minute (0 to 59).
        second : Optional[Union[int, Int]], optional
            Two-digit second (0 to 59).
        millisecond : Optional[Union[int, Int]], optional
            Millisecond (0 to 999).
        variable_name_suffix : str, default ''
            A JavaScript variable name suffix string.
            This setting is sometimes useful for JavaScript's debugging.
        skip_init_substitution_expression_appending : bool, default False
            A boolean indicates whether to skip an initial substitution
            expression or not. The `DateTime` class uses this option internally.

        References
        ----------
        - DateTime class
            - https://simon-ritchie.github.io/apysc/en/datetime.html
        - DateTime class year property
            - https://simon-ritchie.github.io/apysc/en/datetime_year.html
        - DateTime class month property
            - https://simon-ritchie.github.io/apysc/en/datetime_month.html
        - DateTime class day property
            - https://simon-ritchie.github.io/apysc/en/datetime_day.html
        - DateTime class hour property
            - https://simon-ritchie.github.io/apysc/en/datetime_hour.html
        - DateTime class minute property
            - https://simon-ritchie.github.io/apysc/en/datetime_minute.html
        - DateTime class second property
            - https://simon-ritchie.github.io/apysc/en/datetime_second.html
        - DateTime class millisecond property
            - https://simon-ritchie.github.io/apysc/en/datetime_millisecond.html
        - DateTime class weekday_js and weekday_py properties
            - https://simon-ritchie.github.io/apysc/en/datetime_weekday_js_and_weekday_py.html  # noqa
        - DateTime class now interface
            - https://simon-ritchie.github.io/apysc/en/datetime_now.html

        Examples
        --------
        >>> import apysc as ap
        >>> datetime_: ap.DateTime = ap.DateTime(
        ...     year=2022,
        ...     month=12,
        ...     day=5,
        ...     hour=10,
        ...     minute=30,
        ...     second=50,
        ...     millisecond=500,
        ... )
        >>> datetime_.year
        Int(2022)
        >>> datetime_.month
        Int(12)
        >>> datetime_.day
        Int(5)
        >>> datetime_.hour
        Int(10)
        >>> datetime_.minute
        Int(30)
        >>> datetime_.millisecond
        Int(500)
        >>> datetime_.weekday_py
        Int(0)
        >>> datetime_.weekday_js
        Int(1)
        """
        from apysc._expression import expression_variables_util
        from apysc._expression import var_names
        from apysc._expression.event_handler_scope import TemporaryNotHandlerScope

        with TemporaryNotHandlerScope():
            self._variable_name_suffix = variable_name_suffix
            self.variable_name = expression_variables_util.get_next_variable_name(
                type_name=var_names.DATETIME
            )

            self._set_init_year_value(year=year)
            self._set_init_month_value(month=month)
            self._set_init_day_value(day=day)
            self._set_init_hour_value(hour=hour)
            self._set_init_minute_value(minute=minute)
            self._set_init_second_value(second=second)
            self._set_init_millisecond_value(millisecond=millisecond)

            self._append_constructor_expression()

        self._append_initial_substitution_expression_if_in_handler_scope(
            skip_appending=skip_init_substitution_expression_appending,
        )

    @final
    @add_debug_info_setting(module_name=__name__)
    def _append_constructor_expression(self) -> None:
        """
        Append a constructor expression.
        """
        import apysc as ap

        expression: str = self._create_initial_substitution_expression()
        expression = f"var {expression}"
        ap.append_js_expression(expression=expression)

    @final
    def _create_initial_substitution_expression(self) -> str:
        """
        Create an initial value's substitution expression string.

        Returns
        -------
        expression : str
            Created expression string.
        """
        expression: str = f"{self.variable_name} = new Date("
        expression += self._get_init_year_argument_expression()
        expression += self._get_init_month_argument_expression()
        expression += self._get_init_day_argument_expression()
        expression += self._get_init_hour_argument_expression()
        expression += self._get_init_minute_argument_expression()
        expression += self._get_init_second_argument_expression()
        expression += self._get_init_millisecond_argument_expression()
        expression += ");"
        return expression

    @classmethod
    @final
    def now(cls) -> "DateTime":
        """
        Get a `DateTime` instance of the current time.

        Returns
        -------
        dt : DateTime
            A created `DateTime` instance.

        References
        ----------
        - DateTime class now interface
            - https://simon-ritchie.github.io/apysc/en/datetime_now.html

        Examples
        --------
        >>> from datetime import datetime
        >>> import apysc as ap
        >>> py_now: datetime = datetime.now()
        >>> ap_now: ap.DateTime = ap.DateTime.now()
        >>> ap_now.year == py_now.year
        Boolean(True)
        >>> ap_now.month == py_now.month
        Boolean(True)
        >>> ap_now.day == py_now.day
        Boolean(True)
        """
        now_: datetime = datetime.now()
        dt: DateTime = DateTime(year=1970, month=1, day=1)
        dt._year._value = now_.year
        dt._month._value = now_.month
        dt._day._value = now_.day
        dt._hour._value = now_.hour
        dt._minute._value = now_.minute
        dt._second._value = now_.second
        dt._append_now_expression(dt=dt)
        return dt

    def _append_now_expression(self, *, dt: "DateTime") -> None:
        """
        Append a `now` interface expression string.

        Parameters
        ----------
        dt : DateTime
            A target `DateTime` instance.
        """
        import apysc as ap

        expression: str = f"{dt.variable_name} = new Date();"
        ap.append_js_expression(expression=expression)



"""Class implementation for the blank object interface.
The apysc uses this interface at the custom event interface
or something else.
"""

from typing_extensions import final

from apysc._html.debug_mode import add_debug_info_setting


class BlankObjectMixIn:

    _is_blank_object_initialized: bool = False
    _blank_object_variable_name: str

    @final
    @add_debug_info_setting(module_name=__name__)
    def _initialize_blank_object_if_not_initialized(self) -> None:
        """
        Initialize a blank object value if this interface
        does not initialize it yet.
        """
        from apysc._expression import expression_data_util
        from apysc._expression import expression_variables_util
        from apysc._expression import var_names
        from apysc._expression.event_handler_scope import TemporaryNotHandlerScope

        if self._is_blank_object_initialized:
            return
        with TemporaryNotHandlerScope():
            self._blank_object_variable_name = (
                expression_variables_util.get_next_variable_name(
                    type_name=var_names.BLANK_OBJECT
                )
            )
            expression: str = f"var {self._blank_object_variable_name} = {{}};"
            expression_data_util.append_js_expression(expression=expression)
            self._is_blank_object_initialized = True

    @property
    @add_debug_info_setting(module_name=__name__)
    def blank_object_variable_name(self) -> str:
        """
        Get a blank object variable name.

        Returns
        -------
        blank_object_variable_name : str
            A blank object variable name.
        """
        self._initialize_blank_object_if_not_initialized()
        return self._blank_object_variable_name



"""This module is for the translation mapping data of the
following document:

Document file: animation_scale_x_and_y_from_center.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {
    ##################################################
    "# animation_scale_x_from_center and animation_scale_y_from_center interfaces": "# animation_scale_x_from_center と animation_scale_y_from_center のインターフェイス",  # noqa
    ##################################################
    "This page explains the `animation_scale_x_from_center` and `animation_scale_y_from_center` method interfaces.": "このページでは`animation_scale_x_from_center`と`animation_scale_y_from_center`メソッドの各インターフェイスについて説明します。",  # noqa
    ##################################################
    "## What interfaces are these?": "## 各インターフェイスの概要",
    ##################################################
    "The `animation_scale_x_from_center` method interface creates an `ap.AnimationScaleXFromCenter` instance. You can animate x-direction's scale with it.": "`animation_scale_x_from_center`メソッドのインターフェイスは`ap.AnimationScaleXFromCenter`クラスのインスタンスを生成します。そのインスタンスを使って中央座標を基準としたX軸方向の拡縮のアニメーションを設定することができます。",  # noqa
    ##################################################
    "Similarly, the `animation_scale_y_from_center` method interface creates an `ap.AnimationScaleYFromCenter` instance. You can animate y-direction's scale with it.": "同様に`animation_scale_y_from_center`メソッドのインターフェイスでは`ap.AnimationScaleYFromCenter`クラスのインスタンスを生成します。そのインスタンスを使ってY軸方向の拡縮のアニメーションを設定することができます。",  # noqa
    ##################################################
    "These interfaces exist on a `GraphicsBase` subclass (that has the `scale_x_from_center` and `scale_y_from_center` interfaces), such as the `Rectangle` or `Circle`.": "これらのインターフェイスは`scale_x_from_center`や`scale_y_from_center`などのインターフェイスを持つ`Rectangle`や`Circle`などの`GraphicsBase`のサブクラス上に存在します。",  # noqa
    ##################################################
    "## Basic usage": "## 基本的な使い方",
    ##################################################
    "The following example sets the scale-x (left-side rectangle) and scale-y (right-side rectangle) animation (from 1.0 to 2.0) with the `animation_scale_x_from_center` and `animation_scale_y_from_center` methods:": "以下のコード例ではX軸方向の拡縮（1.0から2.0）のアニメーションを`animation_scale_x_from_center`メソッドを使って左側の四角に設定しています。同様にY軸方向の拡縮のアニメーションを右側の四角に設定しています。",  # noqa
    ##################################################
    '```py\n# runnable\nfrom enum import Enum\n\nfrom typing_extensions import TypedDict\n\nimport apysc as ap\n\nDURATION: int = 1000\nEASING: ap.Easing = ap.Easing.EASE_OUT_QUINT\n\n\nclass Direction(Enum):\n    X = 1\n    Y = 2\n\n\nclass Options(TypedDict):\n    direction: Direction\n\n\ndef on_animation_complete_1(\n    e: ap.AnimationEvent[ap.Rectangle], options: Options\n) -> None:\n    """\n    The handler that the animation calls when its end.\n\n    Parameters\n    ----------\n    e : AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    rectangle: ap.Rectangle = e.this.target\n    SCALE: float = 1.0\n    if options["direction"] == Direction.X:\n        rectangle.animation_scale_x_from_center(\n            scale_x_from_center=SCALE,\n            duration=DURATION,\n            easing=EASING,\n        ).animation_complete(\n            on_animation_complete_2,\n            options=options,\n        ).start()\n    elif options["direction"] == Direction.Y:\n        rectangle.animation_scale_y_from_center(\n            scale_y_from_center=SCALE,\n            duration=DURATION,\n            easing=EASING,\n        ).animation_complete(\n            on_animation_complete_2,\n            options=options,\n        ).start()\n\n\ndef on_animation_complete_2(\n    e: ap.AnimationEvent[ap.Rectangle], options: Options\n) -> None:\n    """\n    The handler that the animation calls when its end.\n\n    Parameters\n    ----------\n    e : AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    rectangle: ap.Rectangle = e.this.target\n    SCALE: float = 2.0\n    if options["direction"] == Direction.X:\n        rectangle.animation_scale_x_from_center(\n            scale_x_from_center=SCALE,\n            duration=DURATION,\n            easing=EASING,\n        ).animation_complete(\n            on_animation_complete_1,\n            options=options,\n        ).start()\n    elif options["direction"] == Direction.Y:\n        rectangle.animation_scale_y_from_center(\n            scale_y_from_center=SCALE,\n            duration=DURATION,\n            easing=EASING,\n        ).animation_complete(\n            on_animation_complete_1,\n            options=options,\n        ).start()\n\n\nap.Stage(\n    stage_width=250, stage_height=150, background_color="#333", stage_elem_id="stage"\n)\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color="#0af")\nleft_rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n    x=50, y=50, width=50, height=50\n)\nright_rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n    x=150, y=50, width=50, height=50\n)\n\noptions: Options = {"direction": Direction.X}\nleft_rectangle.animation_scale_x_from_center(\n    scale_x_from_center=2.0,\n    duration=DURATION,\n    easing=EASING,\n).animation_complete(\n    on_animation_complete_1,\n    options=options,\n).start()\n\noptions = {"direction": Direction.Y}\nright_rectangle.animation_scale_y_from_center(\n    scale_y_from_center=2.0,\n    duration=DURATION,\n    easing=EASING,\n).animation_complete(\n    on_animation_complete_1,\n    options=options,\n).start()\n\nap.save_overall_html(dest_dir_path="./animation_scale_x_and_y_from_center_basic_usage/")\n```': '```py\n# runnable\nfrom enum import Enum\n\nfrom typing_extensions import TypedDict\n\nimport apysc as ap\n\nDURATION: int = 1000\nEASING: ap.Easing = ap.Easing.EASE_OUT_QUINT\n\n\nclass Direction(Enum):\n    X = 1\n    Y = 2\n\n\nclass Options(TypedDict):\n    direction: Direction\n\n\ndef on_animation_complete_1(\n    e: ap.AnimationEvent[ap.Rectangle], options: Options\n) -> None:\n    """\n    The handler that the animation calls when its end.\n\n    Parameters\n    ----------\n    e : AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    rectangle: ap.Rectangle = e.this.target\n    SCALE: float = 1.0\n    if options["direction"] == Direction.X:\n        rectangle.animation_scale_x_from_center(\n            scale_x_from_center=SCALE,\n            duration=DURATION,\n            easing=EASING,\n        ).animation_complete(\n            on_animation_complete_2,\n            options=options,\n        ).start()\n    elif options["direction"] == Direction.Y:\n        rectangle.animation_scale_y_from_center(\n            scale_y_from_center=SCALE,\n            duration=DURATION,\n            easing=EASING,\n        ).animation_complete(\n            on_animation_complete_2,\n            options=options,\n        ).start()\n\n\ndef on_animation_complete_2(\n    e: ap.AnimationEvent[ap.Rectangle], options: Options\n) -> None:\n    """\n    The handler that the animation calls when its end.\n\n    Parameters\n    ----------\n    e : AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    rectangle: ap.Rectangle = e.this.target\n    SCALE: float = 2.0\n    if options["direction"] == Direction.X:\n        rectangle.animation_scale_x_from_center(\n            scale_x_from_center=SCALE,\n            duration=DURATION,\n            easing=EASING,\n        ).animation_complete(\n            on_animation_complete_1,\n            options=options,\n        ).start()\n    elif options["direction"] == Direction.Y:\n        rectangle.animation_scale_y_from_center(\n            scale_y_from_center=SCALE,\n            duration=DURATION,\n            easing=EASING,\n        ).animation_complete(\n            on_animation_complete_1,\n            options=options,\n        ).start()\n\n\nap.Stage(\n    stage_width=250, stage_height=150, background_color="#333", stage_elem_id="stage"\n)\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color="#0af")\nleft_rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n    x=50, y=50, width=50, height=50\n)\nright_rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n    x=150, y=50, width=50, height=50\n)\n\noptions: Options = {"direction": Direction.X}\nleft_rectangle.animation_scale_x_from_center(\n    scale_x_from_center=2.0,\n    duration=DURATION,\n    easing=EASING,\n).animation_complete(\n    on_animation_complete_1,\n    options=options,\n).start()\n\noptions = {"direction": Direction.Y}\nright_rectangle.animation_scale_y_from_center(\n    scale_y_from_center=2.0,\n    duration=DURATION,\n    easing=EASING,\n).animation_complete(\n    on_animation_complete_1,\n    options=options,\n).start()\n\nap.save_overall_html(dest_dir_path="./animation_scale_x_and_y_from_center_basic_usage/")\n```',  # noqa
    ##################################################
    "## animation_scale_x_from_center API": "## animation_scale_x_from_center API",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "Set the scale-x from the center point animation setting.<hr>": "中央座標を基準としたX軸の拡縮アニメーションを設定します。<hr>",  # noqa
    ##################################################
    "**[Parameters]**": "**[引数]**",
    ##################################################
    "- `scale_x_from_center`: Number or float": "- `scale_x_from_center`: Number or float",  # noqa
    ##################################################
    "  - The final scale-x of the animation.": "  - X軸の拡縮のアニメーションの最終値。",
    ##################################################
    "- `duration`: Int or int, default 3000": "- `duration`: Int or int, default 3000",
    ##################################################
    "  - Milliseconds before an animation ends.": "  - アニメーション完了までのミリ秒。",
    ##################################################
    "- `delay`: Int or int, default 0": "- `delay`: Int or int, default 0",
    ##################################################
    "  - Milliseconds before an animation starts.": "  - アニメーション開始までの遅延時間のミリ秒。",
    ##################################################
    "- `easing`: Easing, default Easing.LINEAR": "- `easing`: Easing, default Easing.LINEAR",  # noqa
    ##################################################
    "  - Easing setting.": "  - イージング設定。",
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Returns]**": "**[返却値]**",
    ##################################################
    "- `animation_scale_x_from_center`: AnimationScaleXFromCenter": "- `animation_scale_x_from_center`: AnimationScaleXFromCenter",  # noqa
    ##################################################
    "  - Created animation setting instance.": "  - 生成されたアニメーションのインスタンス。",
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Notes]**": "**[特記事項]**",
    ##################################################
    "To start this animation, you need to call the `start` method of the returned instance.<hr>": "アニメーションを開始するには返却されたインスタンスの`start`メソッドを呼び出す必要があります。<hr>",  # noqa
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    '```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color="#0af")\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50\n... )\n>>> _ = rectangle.animation_scale_x_from_center(\n...     scale_x_from_center=0.5,\n...     duration=1500,\n...     easing=ap.Easing.EASE_OUT_QUINT,\n... ).start()\n```': '```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color="#0af")\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50\n... )\n>>> _ = rectangle.animation_scale_x_from_center(\n...     scale_x_from_center=0.5,\n...     duration=1500,\n...     easing=ap.Easing.EASE_OUT_QUINT,\n... ).start()\n```',  # noqa
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[References]**": "**[関連資料]**",
    ##################################################
    "- [Animation interfaces duration setting](https://simon-ritchie.github.io/apysc/en/animation_duration.html)": "- [各アニメーションインターフェイスの duration （アニメーション時間）設定](https://simon-ritchie.github.io/apysc/jp/jp_animation_duration.html)",  # noqa
    ##################################################
    "- [Animation interfaces delay setting](https://simon-ritchie.github.io/apysc/en/animation_delay.html)": "- [各アニメーションインターフェイスの delay （遅延時間）設定](https://simon-ritchie.github.io/apysc/jp/jp_animation_delay.html)",  # noqa
    ##################################################
    "- [Each animation interface return value](https://simon-ritchie.github.io/apysc/en/animation_return_value.html)": "- [各アニメーションインターフェイスの返却値](https://simon-ritchie.github.io/apysc/jp/jp_animation_return_value.html)",  # noqa
    ##################################################
    "- [Sequential animation setting](https://simon-ritchie.github.io/apysc/en/sequential_animation.html)": "- [連続したアニメーション設定](https://simon-ritchie.github.io/apysc/jp/jp_sequential_animation.html)",  # noqa
    ##################################################
    "- [animation_parallel interface](https://simon-ritchie.github.io/apysc/en/animation_parallel.html)": "- [animation_parallel インターフェイス](https://simon-ritchie.github.io/apysc/jp/jp_animation_parallel.html)",  # noqa
    ##################################################
    "- [Easing enum](https://simon-ritchie.github.io/apysc/en/easing_enum.html)": "- [イージングのenum](https://simon-ritchie.github.io/apysc/jp/jp_easing_enum.html)",  # noqa
    ##################################################
    "## animation_scale_y_from_center API": "## animation_scale_y_from_center API",
    ##################################################
    '<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>': '<span class="inconspicuous-txt">特記事項: このAPIドキュメントはドキュメントビルド用のスクリプトによって自動で生成・同期されています。そのためもしかしたらこの節の内容は前節までの内容と重複している場合があります。</span>',  # noqa
    ##################################################
    "**[Interface summary]**": "**[インターフェイス概要]**",
    ##################################################
    "Set the scale-y from the center point animation setting.<hr>": "中央座標を基準としたY軸の拡縮アニメーションを設定します。<hr>",  # noqa
    ##################################################
    "**[Parameters]**": "**[引数]**",
    ##################################################
    "- `scale_y_from_center`: Number or float": "- `scale_y_from_center`: Number or float",  # noqa
    ##################################################
    "  - The final scale-y of the animation.": "  - Y軸の拡縮のアニメーションの最終値。",
    ##################################################
    "- `duration`: Int or int, default 3000": "- `duration`: Int or int, default 3000",
    ##################################################
    "  - Milliseconds before an animation ends.": "  - アニメーション完了までのミリ秒。",
    ##################################################
    "- `delay`: Int or int, default 0": "- `delay`: Int or int, default 0",
    ##################################################
    "  - Milliseconds before an animation starts.": "  - アニメーション開始までの遅延時間のミリ秒。",
    ##################################################
    "- `easing`: Easing, default Easing.LINEAR": "- `easing`: Easing, default Easing.LINEAR",  # noqa
    ##################################################
    "  - Easing setting.": "  - イージング設定。",
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Returns]**": "**[返却値]**",
    ##################################################
    "- `animation_scale_y_from_center`: AnimationScaleYFromCenter": "- `animation_scale_y_from_center`: AnimationScaleYFromCenter",  # noqa
    ##################################################
    "  - Created animation setting instance.": "  - 生成されたアニメーションのインスタンス。",
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[Notes]**": "**[特記事項]**",
    ##################################################
    "To start this animation, you need to call the `start` method of the returned instance.<hr>": "アニメーションを開始するには返却されたインスタンスの`start`メソッドを呼び出す必要があります。<hr>",  # noqa
    ##################################################
    "**[Examples]**": "**[コードサンプル]**",
    ##################################################
    '```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color="#0af")\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50\n... )\n>>> _ = rectangle.animation_scale_y_from_center(\n...     scale_y_from_center=0.5,\n...     duration=1500,\n...     easing=ap.Easing.EASE_OUT_QUINT,\n... ).start()\n```': '```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color="#0af")\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50\n... )\n>>> _ = rectangle.animation_scale_y_from_center(\n...     scale_y_from_center=0.5,\n...     duration=1500,\n...     easing=ap.Easing.EASE_OUT_QUINT,\n... ).start()\n```',  # noqa
    ##################################################
    "<hr>": "<hr>",
    ##################################################
    "**[References]**": "**[関連資料]**",
    ##################################################
    "- [Animation interfaces duration setting](https://simon-ritchie.github.io/apysc/en/animation_duration.html)": "- [各アニメーションインターフェイスの duration （アニメーション時間）設定](https://simon-ritchie.github.io/apysc/jp/jp_animation_duration.html)",  # noqa
    ##################################################
    "- [Animation interfaces delay setting](https://simon-ritchie.github.io/apysc/en/animation_delay.html)": "- [各アニメーションインターフェイスの delay （遅延時間）設定](https://simon-ritchie.github.io/apysc/jp/jp_animation_delay.html)",  # noqa
    ##################################################
    "- [Each animation interface return value](https://simon-ritchie.github.io/apysc/en/animation_return_value.html)": "- [各アニメーションインターフェイスの返却値](https://simon-ritchie.github.io/apysc/jp/jp_animation_return_value.html)",  # noqa
    ##################################################
    "- [Sequential animation setting](https://simon-ritchie.github.io/apysc/en/sequential_animation.html)": "- [連続したアニメーション設定](https://simon-ritchie.github.io/apysc/jp/jp_sequential_animation.html)",  # noqa
    ##################################################
    "- [animation_parallel interface](https://simon-ritchie.github.io/apysc/en/animation_parallel.html)": "- [animation_parallel インターフェイス](https://simon-ritchie.github.io/apysc/jp/jp_animation_parallel.html)",  # noqa
    ##################################################
    "- [Easing enum](https://simon-ritchie.github.io/apysc/en/easing_enum.html)": "- [イージングのenum](https://simon-ritchie.github.io/apysc/jp/jp_easing_enum.html)",  # noqa
}



"""Class implementation for the `_set_handler_data` mix-in.
"""

from typing import Callable
from typing import Dict
from typing import Generic
from typing import Optional
from typing import TypeVar

from typing_extensions import final

from apysc._event.event import Event
from apysc._event.handler import HandlerData
from apysc._validation import arg_validation_decos

_Options = TypeVar("_Options")
_EventClass = TypeVar("_EventClass", bound=Event)


class SetHandlerDataMixIn(Generic[_EventClass]):
    @final
    @arg_validation_decos.handler_args_num(arg_position_index=1)
    @arg_validation_decos.is_builtin_dict(arg_position_index=2)
    @arg_validation_decos.handler_options_type(arg_position_index=3)
    def _set_handler_data(
        self,
        *,
        handler: Callable[[_EventClass, _Options], None],
        handlers_dict: Dict[str, HandlerData[_EventClass]],
        options: Optional[_Options],
    ) -> None:
        """
        Set a handler's data to the given dictionary.

        Parameters
        ----------
        handler : Callable[[_EventClass, _Options], None]
            Callable that this instance calls when dispatching.
        handlers_dict : Dict[str, HandlerData]
            Dictionary that this instance sets a handler's data.
        options : Optional[_Options]
            Optional arguments dictionary that this instance passes
            to a handler.
        """
        from apysc._event.handler import get_handler_name

        name: str = get_handler_name(handler=handler, instance=self)
        if options is None:
            options = {}  # type: ignore
        handler_data: HandlerData[_EventClass] = HandlerData(
            handler=handler, options=options
        )
        handlers_dict[name] = handler_data
